from datetime import datetime
import logging
import src.api.okvendas as api_okvendas
from src.entities.log import Log
import src.api.slack as slack

logger = logging.getLogger()


class OnlineLogger:

    @staticmethod
    def send_log(job_name: str, send_slack: bool, send_api: bool, message: str, log_type: str, job_method: str = 'LOG_ONLINE', api_log_identifier: str = ''):
        if send_slack or bool(send_api):
            slack.post_message(f'{job_name} | {message}')

        if send_api:
            api_okvendas.post_log(Log(f'{job_name} | {message}', datetime.now().isoformat(), api_log_identifier, job_method))

        if log_type == 'info':
            logger.info(f'{job_name} | {message}')
        elif log_type == 'warning':
            logger.warning(f'{job_name} | {message}')
        elif log_type == 'error':
            logger.error(f'{job_name} | {message}')


def send_execution_notification(job_config: dict) -> None:
    logger.info(job_config.get('job_name') + ' | Notificando execucao api okvendas')
    api_okvendas.post_log(Log(f'Oking em execucao desde {job_config.get("execution_start_time")} com {job_config.get("job_qty")} jobs para o cliente {job_config.get("integration_id")}', datetime.now().isoformat(), '', 'NOTIFICACAO'))

# online_logger = OnlineLogger.send_log
# online_logger(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'', 'warning', '')
# online_logger(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'', 'info', '')
# online_logger(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'', 'error', '')
#
#



import logging
from datetime import datetime
from os import path
from src.api import oking
import sys
import src.api.okvendas as api_okvendas
from src.entities.log import Log

global is_connected_oracle_client, client_data, start_time

logging.basicConfig(level=logging.DEBUG,
                    format='[%(levelname)s][%(asctime)s] %(message)s',
                    datefmt='%Y-%m-%d %I:%M:%S')
logger = logging.getLogger()

def print_version():
    f = open(path.abspath('version.txt'), 'r')
    v = f.read()
    print(v)
    f.close()
    exit(0)


def get_token_from_params(args: list) -> (str, bool):
    if len(args) >= 2:
        if args[1] == '--version':
            print_version()

        if len(args) >= 3 and args[2] == '--dev':
            return args[1], True

        return args[1], False
    else:
        logger.error('Informe o token da integracao como parametro')
        exit(1)


token_oking, is_dev = get_token_from_params(sys.argv)

start_time = datetime.now().isoformat()
logger.info('Iniciando oking __init__')
logger.info(f'Ambiente: {"Dev" if is_dev else "Prod"}')
if not is_dev:
    # Consultar dados da integracao do cliente (modulos, tempo de execucao, dados api okvendas)
    client_data = oking.get(f'https://appbuilder.openk.com.br/api/consulta/integracao_oking/filtros?token={token_oking}', None)
    api_okvendas.post_log(Log(f'Oking inicializando {client_data["integracao_id"]}', datetime.now().isoformat(), '', 'INICIALIZACAO'))
else:
    # Consultar dados da integracao do cliente em ambiente node-red local
    client_data = oking.get(f'http://127.0.0.1:1880/api/consulta/integracao_oking/filtros', None)
    api_okvendas.post_log(Log(f'Oking inicializando cliente id {client_data["integracao_id"]}', datetime.now().isoformat(), '', 'INICIALIZACAO'))


if client_data is not None:
    assert client_data['integracao_id'] is not None, 'Id da integracao nao informado (Api Oking)'
    assert client_data['db_type'] is not None, 'Tipo do banco de dados nao informado (Api Oking)'
    assert client_data['host'] is not None, 'Host do banco de dados nao informado (Api Oking)'
    assert client_data['database'] is not None, 'Nome do banco de dados nao informado (Api Oking)'
    assert client_data['user'] is not None, 'Usuario do banco de dados nao informado (Api Oking)'
    assert client_data['password'] is not None, 'Senha do banco de dados nao informado (Api Oking)'
    assert client_data['url_api'] is not None, 'Url da api okvendas nao informado (Api Oking)'
    assert client_data['token_api'] is not None, 'Token da api okvendas nao informado (Api Oking)'
    is_connected_oracle_client = False
else:
    logger.warning(f'Cliente nao configurado no painel oking para o token: {token_oking}')



import json
import logging
from typing import Optional, List
import jsonpickle
import requests
import src
from src.api.entities.cliente import Cliente
from src.api.entities.lista_preco import ListaPreco, ListaPrecoResponse
from src.entities.invoice import Invoice
from src.entities.log import Log
from src.entities.order import Queue, Order
from src.entities.orderb2b import OrderB2B
from src.entities.price import Price
from src.entities.response import CatalogoResponse, PriceResponse, InvoiceResponse, TrackingResponse, StockResponse, ClientResponse
from src.entities.tracking import Tracking
from src.entities.photos_sku import PhotosSku

logger = logging.getLogger()


def obj_dict(obj):
    return obj.__dict__


def object_list_to_dict(obj_list: list):
    lista = []
    for obj in obj_list:
        lista.append(obj.toJSON())
    return lista


def post_produtos(produtos: list):
    try:
        url = f'{src.client_data.get("url_api")}/catalogo/produtos'

        json_produtos = jsonpickle.encode(produtos, unpicklable=False)
        logger.info(f'Enviando produto para api okvendas {json_produtos}')
        response = requests.post(url, json=json.loads(json_produtos), headers={
            'Content-type': 'application/json',
            'Accept': 'text/html',
            'access-token': src.client_data.get('token_api')})

        obj = jsonpickle.decode(response.content)
        result = []
        if 200 <= response.status_code <= 299:
            for res in obj:
                result.append(CatalogoResponse(**res))
        else:
            if type(obj) is list:
                for res in obj:
                    result.append(CatalogoResponse(**res))
            else:
                result.append(CatalogoResponse(**obj))

        return result
    except Exception as e:
        logger.error(f'Erro ao enviar produto para api okvendas {e}', exc_info=True)


def send_stocks(url, body, token):
    logger.debug("POST: {}".format(url))
    try:
        # auth = HTTPBasicAuth('teste@example.com', 'real_password')
        headers = {'Content-type': 'application/json',
                   'Accept': 'text/html',
                   'access-token': token}

        logger.info(body)
        json_stock = jsonpickle.encode(body, unpicklable=False)
        response = requests.put(url, json=json.loads(json_stock), headers=headers)

        if response.ok:
            return response.json(), response.status_code
        else:
            if response.content is not None and response.content != '':
                return response.json(), response.status_code

    except Exception as ex:
        logger.error(str(ex))
        return None, response.status_code


def send_stocks_ud(url, body, token) -> List[StockResponse]:
    logger.debug(f"POST: {url}")
    try:
        headers = {
            'Content-type': 'application/json',
            'Accept': 'application/json',
            'access-token': token
        }

        logger.info(body)
        json_stock = jsonpickle.encode(body, unpicklable=False)
        response = requests.post(url, json=json.loads(json_stock), headers=headers)
        return [StockResponse(**c) for c in jsonpickle.decode(response.content)]

    except Exception as e:
        return [StockResponse([body[0].codigo_erp], "-1", str(e), None)]


def post_prices(url, price: Price, token) -> PriceResponse:
    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'text/html',
                   'access-token': token}

        json_prices = jsonpickle.encode(price, unpicklable=False)
        logger.info(f'{url} - {json_prices}')
        response = requests.put(url, json=json.loads(json_prices), headers=headers)

        obj = jsonpickle.decode(response.content)
        return PriceResponse(**obj)

    except Exception as ex:
        logger.error(str(ex))
        return PriceResponse([price.codigo_erp], 3, str(ex), '', '')


def get_order_queue(url: str, token: str, status: str, limit: int) -> List[Queue]:
    queue = []
    try:
        response = requests.get(url.format(status), headers={'Accept': 'application/json', 'access-token': token},
                                params={'limit': limit})
        if response.ok:
            obj = jsonpickle.decode(response.content)
            for o in obj['fila']:
                queue.append(Queue(**o))
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api okvendas {url}' + str(ex), exc_info=True)

    return queue


def get_order(url: str, token: str, order_id: int) -> Order:
    order = None
    try:
        response = requests.get(url.format(order_id), headers={'Accept': 'application/json', 'access-token': token})
        if response.ok:
            obj = jsonpickle.decode(response.content)
            order = Order(**obj)
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api okvendas {url} - {str(ex)}')
        raise

    return order


def get_order_b2b(url: str, token: str, order_id: int) -> OrderB2B:
    order = None
    try:
        response = requests.get(url.format(order_id), headers={'Accept': 'application/json', 'access-token': token})
        if response.ok:
            obj = jsonpickle.decode(response.content)
            order = OrderB2B(**obj)
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api okvendas {url} - {str(ex)}')
        raise

    return order


def put_order_erp_code(url: str, token: str, order_id: int, order_erp_id: str) -> bool:
    try:
        response = requests.put(url, headers={'Accept': 'application/json', 'access-token': token},
                                params={'id': order_id, 'codigo_erp': order_erp_id})
        if response.ok:
            return True
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
            return False
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api okvendas {url}' + str(ex), exc_info=True)
        return False


def put_client_erp_code(url: str, token: str, body: dict) -> bool:
    try:
        data = jsonpickle.encode(body, unpicklable=False)
        response = requests.put(url, data=json.loads(data),
                                headers={'Accept': 'application/json', 'access-token': token})
        if response.ok:
            return True
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
            return False
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api okvendas {url}' + str(ex), exc_info=True)
        return False


def put_protocol_orders(protocolos: List[str]) -> bool:
    url = src.client_data.get('url_api') + f'/pedido/fila'
    try:
        json_protocolos = jsonpickle.encode(protocolos)
        response = requests.put(url, json=json.loads(json_protocolos), headers={'Accept': 'application/json',
                                                                                'access-token': src.client_data.get(
                                                                                    'token_api')})
        if response.ok:
            return True
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
            return False
    except Exception as ex:
        logger.error(f'Erro ao protocolar pedidos na api okvendas {url}' + str(ex), exc_info=True)
        return False


def post_invoices(url: str, invoice: Invoice, token) -> Optional[InvoiceResponse]:
    """
    Enviar NF de um pedido para api okvendas
    Args:
        url: Url da api okvendas
        invoice: Objeto com os dados da NF
        token: Token de acesso da api okvendas

    Returns:
    None se o envio for sucesso. Caso falhe, um objeto contendo status e descrição do erro
    """
    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': token}

        json_invoice = jsonpickle.encode(invoice, unpicklable=False)
        logger.info(f'{url} - {json_invoice}')
        response = requests.post(url, json=json.loads(json_invoice), headers=headers)
        if response.ok:
            return None
        else:
            err = jsonpickle.decode(str(response.text))
            invoice_response = InvoiceResponse(**err)
            if '_okvendas' in invoice_response.message or '_openkuget' in invoice_response.message:
                invoice_response.message = 'Erro interno no servidor. Entre em contato com o suporte'
            return invoice_response

    except Exception as ex:
        return InvoiceResponse(3, str(ex))


def post_tracking(url: str, tracking: Tracking, token) -> Optional[TrackingResponse]:
    """
    Enviar rastreio de um pedido para api okvendas
    Args:
        url: Url da api okvendas
        tracking: Objeto com os dados do rastreio
        token: Token de acesso da api okvendas

    Returns:
    None se o envio for sucesso. Caso falhe, um objeto contendo status e descrição do erro
    """
    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': token}

        json_tracking = jsonpickle.encode(tracking, unpicklable=False)
        logger.info(f'{url} - {json_tracking}')
        response = requests.post(url, json=json.loads(json_tracking), headers=headers)
        if response.ok:
            return None
        else:
            err = jsonpickle.decode(str(response.text))
            tracking_response = TrackingResponse(**err)
            if '_okvendas' in tracking_response.message or '_openkuget' in tracking_response.message:
                tracking_response.message = 'Erro interno no servidor. Entre em contato com o suporte'
            return tracking_response

    except Exception as ex:
        return TrackingResponse(3, str(ex))


def post_log(log: Log) -> bool:
    try:
        if not src.is_dev:
            url = f'{src.client_data.get("url_api")}/log/logIntegracao'
            headers = {
                'Content-type': 'application/json',
                'Accept': 'application/json',
                'access-token': src.client_data.get('token_api')
            }

            json_log = jsonpickle.encode([log], unpicklable=False)
            print(json_log)
            response = requests.post(url, json=json.loads(json_log), headers=headers)
            if response.ok:
                return True

            return False
        else:
            return True
    except Exception:
        return False


def post_clients(clients: List[Cliente]) -> List[ClientResponse]:
    url = f'{src.client_data.get("url_api")}/cliente/clientes'
    headers = {
        'Content-type': 'application/json',
        'Accept': 'application/json',
        'access-token': src.client_data.get('token_api')
    }

    try:
        payload = jsonpickle.encode(clients, unpicklable=False)
        response = requests.put(url, json=json.loads(payload), headers=headers)
        results = jsonpickle.decode(response.text)
        return [ClientResponse(**r) for r in results]

    except Exception as e:
        return [ClientResponse([c.cpf or c.cnpj for c in clients], 3, str(e))]


def put_photos_sku(body: List[PhotosSku]):
    """
        Cadastra Fotos do SKU
        Podem ser enviadas até 50 fotos por vez.
        Args:
            url: Url da api okvendas
            token: Token de acesso da api okvendas
            photosProduto: Objeto com os dados das Fotos
        Returns:
        True se foi atualizado com Sucesso. False se não foi atualizado
    """
    try:
        url = f'{src.client_data.get("url_api")}/v2/catalogo/fotos/sku'

        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api')}

        json_photos = jsonpickle.encode(body, unpicklable=False)
        logger.info(f'{url} - {json_photos}')

        response = requests.put(url,
                                headers=headers,
                                json=json.loads(json_photos))
        if response.ok:
            return True
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
            return False
    except Exception as ex:
        logger.error(f'Erro ao realizar PUT /v2/catalogo/fotos/sku na api okvendas {url}' + str(ex), exc_info=True)
        return False


def put_price_lists(body: List[ListaPreco]) -> List[ListaPrecoResponse]:
    try:
        url = f'{src.client_data.get("url_api")}/catalogo/listapreco'
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api')}
        json_body = jsonpickle.encode(body, unpicklable=False)

        response = requests.put(url, headers=headers, json=json.loads(json_body))
        if response.ok:
            res = jsonpickle.decode(response.content)
            return [ListaPrecoResponse(**r) for r in res]
        else:
            raise response.content
    except Exception as e:
        logger.error(f'Erro ao realizar PUT /catalogo/listapreco na api okvendas {str(e)}')


def put_price_lists_products(price_list_code: str, body: str) -> bool:
    try:
        url = f'{src.client_data.get("url_api")}/catalogo/listapreco/preco/lote?price_list_code={price_list_code}'
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api')}
        json_body = jsonpickle.encode(body, unpicklable=False)

        response = requests.put(url, headers=headers, json=json.loads(json_body))
        return response.ok
    except Exception as e:
        logger.error(f'Erro ao realizar PUT /catalogo/listapreco/preco/lote na api okvendas {str(e)}')



import time
from datetime import datetime
from typing import List
from src.jobs.system_jobs import OnlineLogger
from src.api.entities.cliente import Cliente, Endereco
from src.database import utils, queries
import logging
import src.database.connection as database
from src.database.entities.client import Client
from src.entities.log import Log
import src.api.okvendas as api_okvendas
from src.database.utils import DatabaseConfig

logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_send_clients(job_config: dict) -> None:
    """
    Job para enviar os clientes para api okvendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    try:
        db_config = utils.get_database_config(job_config)
        db_clients = get_clients(db_config, job_config.get('sql'))

        if len(db_clients) <= 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nenhum cliente retornado para integracao no momento', 'warning', 'CLIENTE')
            return

        clients_inserted = insert_out_clients(db_config, db_clients)
        if not clients_inserted:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nao foi possivel inserir os clientes no banco semaforo', 'error', 'CLIENTE')

        clients = [Cliente(
            nome=c.name,
            razao_social=c.company_name,
            sexo='M',
            data_nascimento=None,
            data_bloqueio=c.blocked_date,
            data_constituicao=None,
            cpf=c.cpf or "",
            cnpj=c.cnpj,
            endereco=Endereco(True, c.address_type, c.address, c.zipcode, c.number, c.complement, c.neighbourhood, c.city, c.state, c.residential_phone or c.mobile_phone, str(), c.reference, 'BR', c.ibge_code),
            email=c.email,
            codigo_referencia=c.client_erp,
            telefone_residencial=c.residential_phone,
            telefone_celular=c.mobile_phone,
            inscricao_estadual=c.state_registration,
            compra_liberada=c.purchase_released,
            site_pertencente=c.belonging_site) for c in db_clients]

        total = len(clients)
        page = 10
        limit = 10 if total > 10 else total
        offset = 0

        partial_clients = clients[offset:limit]
        while limit <= total:
            results = api_okvendas.post_clients(partial_clients)
            if len(results) < 1:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao integrar clientes: {results[0].message}', 'error', 'CLIENTE')

            sucessful_results = [result for result in results if result.status == 1]
            failed_results = [result for result in results if result.status > 1]

            if len(sucessful_results) > 0:
                protocol_clients(db_config, [c.codigo_referencia for sr in sucessful_results for c in clients if sr.identifiers[0].__contains__(c.cpf or c.cnpj)])

            if len(failed_results) > 0:
                for fr in failed_results:
                    for c in partial_clients:
                        time.sleep(0.3)
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao integrar cliente cod.erp {c.codigo_referencia}: {fr.message}', 'error',
                                      'CLIENTE', c.codigo_referencia)

            limit = limit + page
            offset = offset + page
            partial_clients = clients[offset:limit]

    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro nao esperado na integracao de clientes: {str(e)}', 'error', 'CLIENTE')


def get_clients(db_config: DatabaseConfig, sql: str) -> List[Client]:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    cursor.execute(sql)
    results = cursor.fetchall()
    columns = [desc[0] for desc in cursor.description]

    cursor.close()
    conn.close()

    clients_list = []
    new: dict = {}
    for row in results:
        for i, c in enumerate(columns):
            new[c.lower()] = row[i]

        clients_list.append(new.copy())

    clients = [Client(**c) for c in clients_list]
    return clients


def insert_out_clients(db_config: DatabaseConfig, clients: List[Client]) -> bool:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    # cursor.executemany(queries.get_insert_out_clients(db_config.db_type), [queries.get_command_parameter(db_config.db_type, [c.client_erp, datetime.now(), datetime.now()]) for c in clients])
    for c in clients:
        cursor.execute(queries.get_insert_out_clients(db_config.db_type), queries.get_command_parameter(db_config.db_type, [c.client_erp, datetime.now(), datetime.now()]))

    result = cursor.rowcount

    cursor.close()
    conn.commit()
    conn.close()

    return result > 0


def protocol_clients(db_config: DatabaseConfig, client_erp_codes: List[str]) -> bool:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    cursor.executemany(queries.get_out_client_protocol_command(db_config.db_type),
                       [queries.get_command_parameter(db_config.db_type, [c]) for c in client_erp_codes])
    result = cursor.rowcount

    cursor.close()
    conn.commit()
    conn.close()

    return result > 0



import json


class Product:
    def __init__(self, product_dict: dict):
        self.imagem_base64 = product_dict['IMAGEM_BASE64']
        self.codigo_erp = product_dict['CODIGO_ERP']
        self.parceiro = 1 # parceiro 1 = ERP
        self.codigo_referencia = product_dict['CODIGO_REFERENCIA']
        self.agrupador = product_dict['AGRUPADOR']
        self.nome = product_dict['NOME']
        self.descricao = product_dict['DESCRICAO']
        self.quantidade_minima = product_dict['QUANTIDADE_MINIMA']
        self.quantidade_maxima_compra = product_dict['QUANTIDADE_CAIXA']
        self.multiplo = product_dict['MULTIPLO']
        self.metakeyword = product_dict['METAKEYWORD']
        self.metadescription = product_dict['METADESCRIPTION']
        self.meses_garantia = product_dict['MESES_GARANTIA']
        self.modelo = product_dict['MODELO']
        self.tempo_adicional_entrega = product_dict['TEMPO_ADICIONAL_ENTREGA']
        self.ativo = bool(product_dict['ATIVO'])
        self.dimensoes = Dimensao(product_dict['PESO'], product_dict['ALTURA'], product_dict['LARGURA'], product_dict['COMPRIMENTO'], product_dict['VOLUME'])
        self.unidade_medida = UnidadeMedida(product_dict['MEDIDA_SIGLA'], product_dict['MEDIDA_DESCRICAO'], product_dict['MEDIDA_CODIGO'])
        self.ipi = product_dict['IPI']
        
        self.hierarquias_categoria = []
        if product_dict['NOME_CATEGORIA_N1'] is not None:
            self.hierarquias_categoria.append(NivelCategoria(product_dict['NOME_CATEGORIA_N1'], product_dict['CODIGO_CATEGORIA_N1'], None))
        
        if product_dict['NOME_CATEGORIA_N2'] is not None:
            self.hierarquias_categoria.append(NivelCategoria(product_dict['NOME_CATEGORIA_N2'], product_dict['CODIGO_CATEGORIA_N2'], product_dict['CODIGO_CATEGORIA_N1']))
        
        if product_dict['NOME_CATEGORIA_N3'] is not None:
            self.hierarquias_categoria.append(NivelCategoria(product_dict['NOME_CATEGORIA_N3'], product_dict['CODIGO_CATEGORIA_N3'], product_dict['CODIGO_CATEGORIA_N2']))
        
        if product_dict['NOME_CATEGORIA_N4'] is not None:
            self.hierarquias_categoria.append(NivelCategoria(product_dict['NOME_CATEGORIA_N4'], product_dict['CODIGO_CATEGORIA_N4'], product_dict['CODIGO_CATEGORIA_N3']))

        self.fabricante = Fabricante(product_dict['FABRICANTE_DESCRICAO'], product_dict['FABRICANTE_LINK_IMAGEM'], product_dict['FABRICANTE_CODIGO'])
        self.marca = Marca(product_dict['MARCA_DESCRICAO'], product_dict['MARCA_LINK_IMAGEM'], product_dict['MARCA_CODIGO'])
        
        self.atributos_produto = []
        hierarquia_atributo = []
        attr = None
        if product_dict['VARIACAO_OPCAO_1'] is not None and product_dict['VARIACAO_OPCAO_VALOR_1'] is not None:
            if product_dict['VARIACAO_OPCAO_2'] is not None and product_dict['VARIACAO_OPCAO_VALOR_2'] is not None:
                if product_dict['VARIACAO_OPCAO_3'] is not None and product_dict['VARIACAO_OPCAO_VALOR_3'] is not None:
                    attr = Atributo(product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_VALOR_1'], 
                                Atributo(product_dict['VARIACAO_OPCAO_2'], product_dict['VARIACAO_OPCAO_2'], product_dict['VARIACAO_OPCAO_VALOR_2'], 
                                    Atributo(product_dict['VARIACAO_OPCAO_3'], product_dict['VARIACAO_OPCAO_3'], product_dict['VARIACAO_OPCAO_VALOR_3'], None)))
                    # hierarquia_atributo.append(product_dict['VARIACAO_OPCAO_VALOR_3'])
                else:
                    attr = Atributo(product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_VALOR_1'], 
                                Atributo(product_dict['VARIACAO_OPCAO_2'], product_dict['VARIACAO_OPCAO_2'], product_dict['VARIACAO_OPCAO_VALOR_2'], None))
                    # hierarquia_atributo.append(product_dict['VARIACAO_OPCAO_VALOR_2'])
            else:
                attr = Atributo(product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_1'], product_dict['VARIACAO_OPCAO_VALOR_1'], None)
                # hierarquia_atributo.append(product_dict['VARIACAO_OPCAO_VALOR_1'])
                    
            if attr is not None:
                self.atributos_produto.append(attr)

        self.preco_estoque = [PrecoEstoque(product_dict['CODIGO_ERP_VARIACAO'], product_dict['SKU_VARIACAO'],
                                            None, product_dict['EAN_13_VARIACAO'], product_dict['EAN_14_VARIACAO'],
                                            product_dict['PRECO_ATUAL'], product_dict['PRECO_LISTA'], product_dict['PRECO_CUSTO'],
                                            product_dict['QUANTIDADE'], self.dimensoes, hierarquia_atributo)]
        
        
        self.outras_descricoes = []
        if product_dict['ADICIONA_DESCRICAO_1'] is not None and product_dict['ADICIONA_CONTEUDO_1'] is not None:
            self.outras_descricoes.append(InfoDescricao(product_dict['ADICIONA_DESCRICAO_1'], product_dict['ADICIONA_CONTEUDO_1'], 1))


    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
            sort_keys=True, indent=4)
        

    def __repr__(self):
        return "<Produto id:%s, codigo_sku:%s>" % (self.id, self.codigo_sku)

    def __str__(self):
        return 'Produto id : %s, ' \
            'SKU : %s' % (self.id, self.codigo_sku)


class PrecoEstoque:
    def __init__(self, codigo_erp_atributo: str, codigo_referencia_atributo: str, codigo_erp_restock: str,
                 codigo_ean: str, codigo_ean14: str, preco: float, preco_lista: float, preco_custo: float, 
                 estoque: int, dimensoes, hierarquia_codigo_atributo):
        self.codigo_erp_atributo = codigo_erp_atributo
        self.codigo_referencia_atributo = codigo_referencia_atributo
        self.codigo_erp_restock = codigo_erp_restock
        self.codigo_ean = codigo_ean
        self.codigo_ean14 = codigo_ean14
        self.preco = preco
        self.preco_lista = preco_lista
        self.preco_custo = preco_custo
        self.estoque = estoque
        self.dimensoes = dimensoes
        self.hierarquia_codigo_atributo = hierarquia_codigo_atributo


class Dimensao:
    def __init__(self, peso, altura, largura, comprimento, volume):
        self.peso = peso
        self.altura = altura
        self.largura = largura
        self.comprimento = comprimento
        self.volume = volume
        

class ItemBrinde:
    def __init__(self, preco_atual, quantidade, desconto, produto_id,
                 codigo_externo, identificador_produto, codigo_referencia,
                 nome_produto, unidade_produto, identificador_atributo,
                 identificador_opcao_atributo, filial_expedicao,
                 filial_faturamento):
        self.preco_atual = preco_atual
        self.quantidade = quantidade
        self.desconto = desconto
        self.produto_id = produto_id
        self.codigo_externo = codigo_externo
        self.identificador_produto = identificador_produto
        self.codigo_referencia = codigo_referencia
        self.nome_produto = nome_produto
        self.unidade_produto = unidade_produto
        self.identificador_atributo = identificador_atributo
        self.identificador_opcao_atributo = identificador_opcao_atributo
        self.filial_expedicao = filial_expedicao
        self.filial_faturamento = filial_faturamento


class ItemPersonalizado:
    def __init__(self, id, preco_liquido, preco_bruto, texto):
        self.id = id
        self.preco_liquido = preco_liquido
        self.preco_bruto = preco_bruto
        self.texto = texto


def customProdutoDecoder(produtoDict):
    return Product(**produtoDict)


class Medida:
    def __init__(self, volume, peso, altura, largura, comprimento):
        self.volume = volume
        self.peso = peso
        self.altura = altura
        self.largura = largura
        self.comprimento = comprimento


class UnidadeMedida:
    def __init__(self, sigla, descricao, codigo):
        self.sigla = sigla
        self.descricao = descricao
        self.codigo = codigo


class NivelCategoria:
    def __init__(self, nome, codigo, codigo_categoria_pai):
        self.nome = nome
        self.codigo = codigo
        self.codigo_categoria_pai = codigo_categoria_pai


class Fabricante:
    def __init__(self, nome, link_imagem, codigo):
        self.nome = nome
        self.link_imagem = link_imagem
        self.codigo = codigo


class Marca:
    def __init__(self, nome, link_imagem, codigo):
        self.nome = nome
        self.link_imagem = link_imagem
        self.codigo = codigo


class Atributo:
    def __init__(self, classificacao, opcao_descricao, opcao_codigo, atributo_pai):
        self.classificacao = classificacao
        self.opcao_descricao = opcao_descricao
        self.opcao_codigo = opcao_codigo
        self.atributo_pai = atributo_pai


class InfoDescricao:
    def __init__(self, descricao, conteudo, ordem):
        self.descricao = descricao
        self.conteudo = conteudo
        self.ordem = ordem


class InfoCaracteristica:
    def __init__(self, nome, valor):
        self.nome = nome
        self.valor = valor


class Estoque:
    def __init__(self, codigo_erp, quantidade):
        self.codigo_erp = codigo_erp
        self.quantidade = quantidade


class EstoqueUnidadeDistribuicao:
    def __init__(self, u_distribuicao, codigo_erp, produto_id,
                 total, reserva, protocolo, parceiro):
        self.unidade_distribuicao = u_distribuicao
        self.codigo_erp = codigo_erp
        self.produto_id = produto_id
        self.quantidade_total = total
        self.quantidade_reserva = reserva
        self.protocolo = protocolo
        self.parceiro = parceiro


class Preco:
    def __init__(self, codigo_erp_atributo, codigo_referencia_atributo,
                 codigo_erp_restock, codigo_ean, codigo_ean14, ativo,
                 preco, preco_lista, preco_custo, estoque):
        self.codigo_erp_atributo = codigo_erp_atributo
        self.codigo_referencia_atributo = codigo_referencia_atributo
        self.codigo_erp_restock = codigo_erp_restock
        self.codigo_ean = codigo_ean
        self.codigo_ean14 = codigo_ean14
        self.ativo = ativo
        self.preco = preco
        self.preco_lista = preco_lista
        self.preco_custo = preco_custo
        self.estoque = estoque
        self.dimensoes = None  # {Medida}
        self.hierarquia_codigo_atributo = []  # "string"



import requests
import logging

logger = logging.getLogger()


def register_warn(message):
    logger.warning(message)
    post_message(message)


def register_error(erro):
    logger.error(erro)
    post_message(erro)


def post_message(message):
    dct_body = {
        'text': message,
        'channel': '#oking'
    }

    url = 'https://hooks.slack.com/services/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'

    try:
        headers = {
            'Content-type': 'application/json',
            'Accept': 'text/html'
        }

        response = requests.post(url, json=dct_body, headers=headers)
        if not 200 <= response.status_code <= 299:
            msg = 'Erro ao enviar slack ' + str(response.status_code) + ' ' + str(dct_body)
            logger.error(msg)
    except Exception:
        logger.error('Erro ao enviar slack')



from datetime import datetime


class Client:
    def __init__(self, nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia,
                 cliente_erp, direcao, data_sincronizacao, codigo_ibge, inscricao_estadual, compra_liberada, site_pertencente, data_bloqueio):
        self.belonging_site = site_pertencente
        self.blocked_date = data_bloqueio
        self.name: str = nome
        self.company_name: str = razao_social
        self.cpf: str = cpf
        self.cnpj: str = cnpj
        self.email: str = email
        self.residential_phone: str = telefone_residencial
        self.mobile_phone: str = telefone_celular
        self.zipcode: str = cep
        self.address_type: str = tipo_logradouro
        self.address: str = logradouro
        self.number: str = numero
        self.complement: str = complemento
        self.neighbourhood: str = bairro
        self.city: str = cidade
        self.state: str = estado
        self.reference: str = referencia
        self.client_erp: str = cliente_erp
        self.direction: str = direcao
        self.sync_date: datetime = data_sincronizacao
        self.ibge_code: str = codigo_ibge
        self.state_registration: str = inscricao_estadual
        self.purchase_released: bool = compra_liberada



from datetime import datetime


class Endereco:
    ativo: bool
    tipo_logradouro: str
    endereco: str
    cep: str
    numero: str
    complemento: str
    bairro: str
    cidade: str
    estado: str
    telefone_contato: str
    descricao_endereco: str
    referencia_entrega: str
    pais: str
    codigo_ibge: str

    def __init__(self, ativo: bool, tipo_logradouro: str, endereco: str, cep: str, numero: str, complemento: str, bairro: str, cidade: str, estado: str, telefone_contato: str, descricao_endereco: str,
                 referencia_entrega: str, pais: str, codigo_ibge: str) -> None:
        self.ativo = ativo
        self.tipo_logradouro = tipo_logradouro
        self.endereco = endereco
        self.cep = cep
        self.numero = numero
        self.complemento = complemento
        self.bairro = bairro
        self.cidade = cidade
        self.estado = estado
        self.telefone_contato = telefone_contato
        self.descricao_endereco = descricao_endereco
        self.referencia_entrega = referencia_entrega
        self.pais = pais
        self.codigo_ibge = codigo_ibge


class Cliente:
    nome: str
    razao_social: str
    sexo: str
    data_nascimento: datetime
    data_constituicao: str
    cpf: str
    cnpj: str
    rg: str
    orgao: str
    email: str
    senha: str
    senha_expirada: bool
    telefone_residencial: str
    telefone_celular: str
    codigo_referencia: str
    inscricao_estadual: str
    inscricao_municipal: str
    limite_credito: int
    data_bloqueio: datetime
    parcelamento: bool
    tipo_pessoa: str
    tipo_usuario: int
    codigo_livro: str
    compra_liberada: bool
    contribuinte_icms: int
    optante_simples_nacional: bool
    fator_juros: int
    fator_frete: int
    fator_cliente: int
    solicitacao_limite_credito: int
    quantidade_lojas: int
    valida_pedido_minimo: bool
    codigo_suframa: str
    eh_cif: bool
    site_pertencente: str
    ramo_atividade: str
    segmento: str
    grupo_economico: str
    endereco: Endereco
    protocolado: bool

    def __init__(self, nome: str, razao_social: str, sexo: str, cpf: str, cnpj: str, email: str, endereco: Endereco, telefone_residencial: str, telefone_celular: str, codigo_referencia: str,
                 orgao: str = str(), senha: str = str(), rg: str = str(), senha_expirada: bool = False, inscricao_estadual: str = str(), inscricao_municipal: str = str(), limite_credito: int = 0,
                 data_bloqueio: datetime = None, parcelamento: bool = False, tipo_pessoa: str = str(), tipo_usuario: int = 0, codigo_livro: str = str(), compra_liberada: bool = False,
                 contribuinte_icms: int = 0, optante_simples_nacional: bool = False, fator_juros: int = 0, fator_frete: int = 0, fator_cliente: int = 0, solicitacao_limite_credito: int = 0,
                 quantidade_lojas: int = 0, valida_pedido_minimo: bool = False, codigo_suframa: str = str(), eh_cif: bool = False, site_pertencente: str = str(), ramo_atividade: str = str(),
                 segmento: str = str(), grupo_economico: str = str(), protocolado: bool = False, data_nascimento: datetime = None, data_constituicao: str = None) -> None:
        self.nome = nome
        self.razao_social = razao_social
        self.sexo = sexo
        self.data_nascimento = data_nascimento
        self.data_constituicao = data_constituicao
        self.cpf = cpf
        self.cnpj = cnpj
        self.rg = rg
        self.orgao = orgao
        self.email = email
        self.senha = senha
        self.senha_expirada = senha_expirada
        self.telefone_residencial = telefone_residencial
        self.telefone_celular = telefone_celular
        self.codigo_referencia = codigo_referencia
        self.inscricao_estadual = inscricao_estadual
        self.inscricao_municipal = inscricao_municipal
        self.limite_credito = limite_credito
        self.data_bloqueio = data_bloqueio
        self.parcelamento = parcelamento
        self.tipo_pessoa = tipo_pessoa
        self.tipo_usuario = tipo_usuario
        self.codigo_livro = codigo_livro
        self.compra_liberada = compra_liberada
        self.contribuinte_icms = contribuinte_icms
        self.optante_simples_nacional = optante_simples_nacional
        self.fator_juros = fator_juros
        self.fator_frete = fator_frete
        self.fator_cliente = fator_cliente
        self.solicitacao_limite_credito = solicitacao_limite_credito
        self.quantidade_lojas = quantidade_lojas
        self.valida_pedido_minimo = valida_pedido_minimo
        self.codigo_suframa = codigo_suframa
        self.eh_cif = eh_cif
        self.site_pertencente = site_pertencente
        self.ramo_atividade = ramo_atividade
        self.segmento = segmento
        self.grupo_economico = grupo_economico
        self.endereco = endereco
        self.protocolado = protocolado



from typing import List


class CatalogoResponse:
    def __init__(self, Message: str, Status: int = 3, codigo_erp: str = '', protocolo: str = '', loja: str = '') -> None:
        self.codigo_erp = codigo_erp
        self.status = Status
        self.message = Message
        self.protocolo = protocolo
        self.loja = loja


class PriceResponse:
    def __init__(self, codigo_erp, Status, Message, protocolo, loja='', codigo_externo_campanha=''):
        self.codigo_erp = codigo_erp
        self.status = Status
        self.message = Message
        self.protocolo = protocolo


class InvoiceResponse:
    def __init__(self, Status: int, Message: str):
        self.status = Status
        self.message = Message


class TrackingResponse:
    def __init__(self, Status: int, Message: str):
        self.status = Status
        self.message = Message


class StockResponse:
    def __init__(self, Identifiers, Status, Message, Protocolo):
        self.identifiers: List[str] = Identifiers
        self.status: int = Status
        self.message: str = Message
        self.protocol: str = Protocolo

class ClientResponse:
    def __init__(self, Identifiers: List[str], Status: int, Message: str, Protocolo):
        self.identifiers: List[str] = Identifiers
        self.status: int = Status
        self.message: str = Message




from datetime import datetime
import logging
import src.database.connection as database
import src.database.utils as utils
from src.jobs.system_jobs import OnlineLogger
from src.database.utils import DatabaseConfig
import src.api.okvendas as api_okvendas
from src.api import slack
from src.database import queries
import src
import time

logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_insert_stock_semaphore(job_config_dict: dict):
    """
    Job para inserir estoques no banco semáforo
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, 'Comando sql para inserir estoques no semaforo nao encontrado', 'warning', 'ESTOQUE')
    else:
        db = database.Connection(db_config)
        conn = db.get_conect()
        cursor = conn.cursor()

        try:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Inserindo estoques no banco semaforo \n {db_config.sql}', 'info', 'ESTOQUE')
            cursor.execute(db_config.sql)
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, '{cursor.rowcount} estoques inseridos no banco semaforo', 'info', 'ESTOQUE')
        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao inserir estoques no banco semaforo {str(ex)}', 'error', 'ESTOQUE')

        cursor.close()
        conn.commit()
        conn.close()


def job_send_stocks(job_config_dict: dict):
    """
    Job para realizar a atualização dos estoques padrão
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    stocks = query_stocks(job_config_dict, db_config)
    atualizados = []
    if len(stocks) or 0 > 0:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, 'Total de estoques a serem atualizados: {len(stocks)}', 'info', '')
        for stock in stocks:
            try:
                time.sleep(1)
                response, status_code = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoque', stock, src.client_data.get('token_api'))

                if response is not None:
                    conexao = database.Connection(db_config)
                    conn = conexao.get_conect()
                    cursor = conn.cursor()

                    codigo_erp = response['codigo_erp']
                    if response['Status'] == 1 or response['Status'] == 'Success':
                        cursor.execute(queries.get_stock_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                        atualizados.append(response['codigo_erp'])
                    elif (response['Status'] == 'Error' or response['Status'] == 3) and response['Message'] == "Erro ao atualizar o estoque. Produto não existente":
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}', 'warning',
                                      'ESTOQUE', codigo_erp)
                        cursor.execute(queries.get_stock_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                    else:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}', 'warning',
                                      'ESTOQUE', codigo_erp)

                    cursor.close()
                    conn.commit()
                    conn.close()
            except Exception as e:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao atualizar estoque do sku {stock.get("codigo_erp")}: {str(e)}', 'error', 'ESTOQUE',
                              stock.get('codigo_erp'))

        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Atualizado estoques no semaforo: {len(atualizados) or 0}', 'info', 'ESTOQUE')
    else:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao ha produtos para atualizar estoque no momento', 'warning', 'ESTOQUE')


def job_send_stocks_ud(job_config_dict: dict):
    """
    Job para realizar a atualização dos estoques por unidade de distribuição
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    stocks = query_stocks_ud(job_config_dict, db_config)
    p_size = len(stocks) if stocks is not None else 0

    if p_size > 0:
        time.sleep(0.5)
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Total de produtos para atualizar estoque: {p_size}', 'info', 'ESTOQUE')
        count = 0
        for stock in stocks:
            codigo_erp = stock["codigo_erp"]
            try:
                response = api_okvendas.send_stocks_ud(src.client_data.get('url_api') + '/catalogo/estoqueUnidadeDistribuicao', [stock], src.client_data.get('token_api'))
                for res in response:
                    if res.status == 1:
                        protocol_stock(job_config_dict, db_config, codigo_erp)
                        count = count + 1
                    elif (res.status == 'Error' or res.status > 1) and res.message.__contains__("não encontrado"):
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao atualizar estoque {codigo_erp} erro da api: {res.message}', 'warning',
                                      'ESTOQUE')
                        protocol_stock(job_config_dict, db_config, codigo_erp)
                    else:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao atualizar estoque do sku {codigo_erp}: {res.message}', 'error', 'ESTOQUE',
                                      codigo_erp)
            except Exception as e:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao atualizar estoque para o sku {stock.get("codigo_erp")}: {str(e)}', 'error',
                              'ESTOQUE', stock.get('codigo_erp'))

        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produtos protocolados no semaforo: {count}', 'info', 'ESTOQUE')
    else:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao ha produtos para atualizar estoque', 'warning', 'ESTOQUE')


def query_stocks_ud(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização por unidade de distribuição
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    stocks = None
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Query estoque de produtos nao configurada', 'warning', 'ESTOQUE')
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                stocks = stock_ud_dict(results)

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{str(ex)}', 'error', 'ESTOQUE')

    return stocks


def query_stocks(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização padrão
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    produtos = None
    if db_config.sql is None or db_config.sql == '':
        slack.register_warn("Query estoque de produtos nao configurada!")
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Query estoque de produtos nao configurada', 'warning', 'ESTOQUE')
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                produtos = stock_dict(results)

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{str(ex)}', 'error', 'ESTOQUE')

    return produtos


def stock_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'codigo_erp': str(row['CODIGO_ERP']),
            'quantidade': int(row['QUANTIDADE'])
        }
        lista.append(pdict)

    return lista


def stock_ud_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'unidade_distribuicao': row['CODIGO_UNIDADE_DISTRIBUICAO'],
            'codigo_erp': row['CODIGO_ERP'],
            'quantidade_total': int(row['QUANTIDADE']),
            'parceiro': 1
        }
        lista.append(pdict)

    return lista


def protocol_stock(job_config_dict, db_config: DatabaseConfig, codigo_erp):
    try:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Protocolando produto {codigo_erp} no semaforo', 'info', 'ESTOQUE')
        conn = database.Connection(db_config).get_conect()
        cursor = conn.cursor()
        cursor.execute(queries.get_stock_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [codigo_erp]))
        cursor.close()
        conn.commit()
        conn.close()
    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao protocolar estoque do sku {codigo_erp}: {str(e)}', 'error', '')



from typing import List, Dict


class FormaEnvioParceiro:
    tracking_code: str
    tracking_type: str
    shipping_type: str
    shipping_status: str
    post_forecast: str
    shipping_mode: str
    plp: str
    route: str
    mega_route: str

    def __init__(self, codigo_rastreio: str, forma_envio: str, tipo_envio: str, status_envio: str, data_previsao_postagem: str, modo_envio: str, plp: str, rota: str, mega_rota: str) -> None:
        self.tracking_code = codigo_rastreio
        self.tracking_type = forma_envio
        self.shipping_type = tipo_envio
        self.shipping_status = status_envio
        self.post_forecast = data_previsao_postagem
        self.shipping_mode = modo_envio
        self.plp = plp
        self.route = rota
        self.mega_route = mega_rota


class FormaPagamentoParceiro:
    payment_type: str
    flag: str
    authorization_code: str
    status_code: str
    transaction_code: str
    created_date: str
    approved_date: str
    status_detail: str
    status: str
    freight_value: int
    installment_value: int
    order_amount: int
    total_paid_amount: int
    installments: int

    def __init__(self, tipo_pagamento: str, bandeira: str, codigo_autorizacao: str, codigo_status: str, codigo_transacao: str, data_criacao: str, data_aprovacao: str, detalhe_status: str,
                 status: str, valor_frete: int, valor_parcela: int, valor_pedido: int, valor_total_pago: int, parcelas: int) -> None:
        self.payment_type = tipo_pagamento
        self.flag = bandeira
        self.authorization_code = codigo_autorizacao
        self.status_code = codigo_status
        self.transaction_code = codigo_transacao
        self.created_date = data_criacao
        self.approved_date = data_aprovacao
        self.status_detail = detalhe_status
        self.status = status
        self.freight_value = valor_frete
        self.installment_value = valor_parcela
        self.order_amount = valor_pedido
        self.total_paid_amount = valor_total_pago
        self.installments = parcelas


class Item:
    main_sku: str
    variation_sku: str
    reference_sku: str
    variation_hierarchy: str
    is_restock: bool
    restock_erp_code: str
    ean: str
    measure_unity: str
    quantity: int
    amount: int
    discount_amount: int
    height: int
    length: int
    width: int
    weight: int
    volume: int
    expedition_branch: str
    invoice_branch: str
    selling_branch_cnpj: str
    branch_erp_code: str
    branch: str

    def __init__(self, sku_principal: str, sku_variacao: str, sku_reference: str, hierarquia_variacao: str, is_restock: bool, codigo_externo_restock: str, ean: str, unidade_medida: str, quantidade: int, value: int,
                 valor_desconto: int, altura: int, comprimento: int, largura: int, peso: int, volume: int, filial_expedicao: str, filial_faturamento: str, cnpj_filial_venda: str,
                 codigo_externo_filial: str, filial: str) -> None:
        self.main_sku = sku_principal
        self.erp_code = sku_variacao
        self.sku = sku_reference
        self.variation_hierarchy = hierarquia_variacao
        self.is_restock = is_restock
        self.restock_erp_code = codigo_externo_restock
        self.ean = ean
        self.measure_unity = unidade_medida
        self.quantity = quantidade
        self.value = value
        self.discount = valor_desconto
        self.height = altura
        self.length = comprimento
        self.width = largura
        self.weight = peso
        self.volume = volume
        self.expedition_branch = filial_expedicao
        self.invoice_branch = filial_faturamento
        self.selling_branch_cnpj = cnpj_filial_venda
        self.branch_erp_code = codigo_externo_filial
        self.branch = filial


class ItensBrinde:
    price: int
    quantity: int
    discount: int
    product_id: str
    erp_code: str
    product_identifier: str
    reference_code: str
    name: str
    unity: str
    attribute_identifier: str
    attribute_option_identifier: str
    expedition_branch: str
    invoice_branch: str

    def __init__(self, preco_atual: int, quantidade: int, desconto: int, produto_id: str, codigo_externo: str, identificador_produto: str, codigo_referencia: str, nome_produto: str,
                 unidade_produto: str, identificador_atributo: str, identificador_opcao_atributo: str, filial_expedicao: str, filial_faturamento: str) -> None:
        self.price = preco_atual
        self.quantity = quantidade
        self.discount = desconto
        self.product_id = produto_id
        self.erp_code = codigo_externo
        self.product_identifier = identificador_produto
        self.reference_code = codigo_referencia
        self.name = nome_produto
        self.unity = unidade_produto
        self.attribute_identifier = identificador_atributo
        self.attribute_option_identifier = identificador_opcao_atributo
        self.expedition_branch = filial_expedicao
        self.invoice_branch = filial_faturamento


class ItensPersonalizado:
    id: str
    net_price: int
    gross_price: int
    text: str

    def __init__(self, id: str, preco_liquido: int, preco_bruto: int, texto: str) -> None:
        self.id = id
        self.net_price = preco_liquido
        self.gross_price = preco_bruto
        self.text = texto


class Titulo:
    installment: int
    amount: int
    expiration_date: str
    discharge_date: str
    paid_date: str
    return_code: str
    return_description: str
    designed_rebate_amount: int
    designed_discount: int
    iof_collected_amount: int
    credited_amount: int
    withdrawn_amount: int
    other_amount: int
    other_credits: int

    def __init__(self, parcela: int, valor: int, data_vencimento: str, data_baixa: str, data_pago: str, codigo_retorno: str, descricao_retorno: str, valor_abatimento_concebido: int,
                 desconto_concebido: int, valor_iof_recolhido: int, valor_creditado: int, valor_sacado: int, outros_valores: int, outros_creditos: int) -> None:
        self.installment = parcela
        self.amount = valor
        self.expiration_date = data_vencimento
        self.discharge_date = data_baixa
        self.paid_date = data_pago
        self.return_code = codigo_retorno
        self.return_description = descricao_retorno
        self.designed_rebate_amount = valor_abatimento_concebido
        self.designed_discount = desconto_concebido
        self.iof_collected_amount = valor_iof_recolhido
        self.credited_amount = valor_creditado
        self.withdrawn_amount = valor_sacado
        self.other_amount = outros_valores
        self.other_credits = outros_creditos


class Pagamento:
    payment_option: str
    installments: int
    flag: str
    rsvarejo_finance_table: str
    rsvarejo_sale_type: str
    channel_id: str
    channel: str
    coupom_number: str
    coupom_amount: int
    purchase_code: str
    channel_order_code: str
    movement_date: str
    market_code: str
    bonds: List[Titulo]
    erp_payment_condition: str

    def __init__(self, opcao_pagamento: str, parcelas: int, bandeira: str, tabela_financiamento_rsvarejo: str, tipo_venda_rsvarejo: str, canal_venda_id: str, canal_venda: str, numero_cupom: str,
                 valor_cupom: int, codigo_compra: str, codigo_pedido_canal: str, data_movimento: str, codigo_mercado: str, titulos: List[Dict], condicao_pagamento_erp: str) -> None:
        self.payment_option = opcao_pagamento
        self.installments = parcelas
        self.flag = bandeira
        self.rsvarejo_finance_table = tabela_financiamento_rsvarejo
        self.rsvarejo_sale_type = tipo_venda_rsvarejo
        self.channel_id = canal_venda_id
        self.channel = canal_venda
        self.coupom_number = numero_cupom
        self.coupom_amount = valor_cupom
        self.purchase_code = codigo_compra
        self.channel_order_code = codigo_pedido_canal
        self.movement_date = data_movimento
        self.market_code = codigo_mercado
        self.bonds = [Titulo(**t) for t in titulos]
        self.erp_payment_condition = condicao_pagamento_erp


class PedidoNotaFiscal:
    invoice_id: int
    oder_id: int
    tax_number: str
    series_number: str
    access_key: str
    total_amount: int
    emission_date: str
    register_date: str
    xml: str
    link_danfe: str
    link_xml: str
    volume_quantity: str

    def __init__(self, nota_fiscal_id: int, pedido_id: int, num_fiscal: str, num_serie: str, access_key: str, valor_total_nota: int, data_emissao: str, dt_cadastro: str, xml: str,
                 link_danfe: str, link_xml: str, quantidade_volume: str) -> None:
        self.invoice_id = nota_fiscal_id
        self.oder_id = pedido_id
        self.tax_number = num_fiscal
        self.series_number = num_serie
        self.access_key = access_key
        self.total_amount = valor_total_nota
        self.emission_date = data_emissao
        self.register_date = dt_cadastro
        self.xml = xml
        self.link_danfe = link_danfe
        self.link_xml = link_xml
        self.volume_quantity = quantidade_volume


class Transacao:
    transaction_id: str
    authorization_number: str
    nsu: str
    return_message: str
    additional_3: str
    additional_4: str

    def __init__(self, transacao_id: str, numero_autorizacao: str, nsu: str, mensagem_retorno: str, adicional_3: str, adicional_4: str) -> None:
        self.transaction_id = transacao_id
        self.authorization_number = numero_autorizacao
        self.nsu = nsu
        self.return_message = mensagem_retorno
        self.additional_3 = adicional_3
        self.additional_4 = adicional_4


class TransportadoraFob:
    name: str
    company_name: str
    cnpj: str

    def __init__(self, nome: str, razao_social: str, cnpj: str) -> None:
        self.name = nome
        self.company_name = razao_social
        self.cnpj = cnpj


class Address:
    zipcode: str
    address_line: str
    number: str
    complement: str
    neighbourhood: str
    city: str
    ibge_code: str
    state: str
    country: str
    reference: str
    description: str
    address_type: str

    def __init__(self, cep: str, logradouro: str, numero: str, complemento: str, bairro: str, cidade: str, codigo_ibge: str, estado: str, pais: str, referencia: str, descricao: str,
                 tipo_logradouro: str) -> None:
        self.zipcode = cep
        self.address_line = logradouro
        self.number = numero
        self.complement = complemento
        self.neighbourhood = bairro or ''
        self.city = cidade
        self.ibge_code = codigo_ibge
        self.state = estado
        self.country = pais
        self.reference = referencia
        self.description = descricao
        self.address_type = tipo_logradouro or 'Rua'


class Usuario:
    erp_code: str
    id: int
    name: str
    company_name: str
    cpf: str
    rg: str
    birth_date: str
    cnpj: str
    sex: str
    email: str
    organ: str
    state_registry: str
    residential_phone: str
    mobile_phone: str
    address: Address
    delivery_address: Address

    def __init__(self, codigo_referencia: str, id: int, nome: str, razao_social: str, cpf: str, rg: str, data_nascimento: str, cnpj: str, sexo: str, email: str, orgao: str,
                 RegistroEstadual: str, TelefoneResidencial: str, TelefoneCelular: str, Endereco: Dict, EnderecoEntrega: Dict) -> None:
        self.erp_code = codigo_referencia
        self.id = id
        self.name = nome
        self.company_name = razao_social
        self.cpf = cpf
        self.rg = rg
        self.birth_date = data_nascimento
        self.cnpj = cnpj
        self.sex = sexo
        self.email = email
        self.organ = orgao
        self.state_registry = RegistroEstadual
        self.residential_phone = TelefoneResidencial
        self.mobile_phone = TelefoneCelular
        self.address = Address(**Endereco)
        self.delivery_address = Address(**EnderecoEntrega)


class OrderB2B:
    order_id: int
    sale_order_id: int
    order_date: str
    generated_date: str
    erp_code: str
    total_amount: int
    payment_method_amount: int
    discount_amount: int
    freight_amount: int
    status: str
    bonds_quantity: int
    delivery_forecast: str
    tracking_code: str
    channel_id: int
    carrier_id: str
    carrier: str
    service_id: int
    service: str
    freight_type: str
    fob_carrier: TransportadoraFob
    transaction: Transacao
    user: Usuario
    payments: List[Pagamento]
    items: List[Item]
    gift_items: List[ItensBrinde]
    custom_items: List[ItensPersonalizado]
    partner_payment_type: FormaPagamentoParceiro
    partner_shipping_method: FormaEnvioParceiro
    invoices: List[PedidoNotaFiscal]
    protocol: str
    erp_representative: str

    def __init__(self, id: int, pedido_venda_id: int, data_pedido: str, data_geracao: str, codigo_referencia: str, valor_total: int, valor_forma_pagamento: int, valor_desconto: int,
                 valor_frete: int, status: str, quantidade_titulos: int, previsao_entrega: str, codigo_rastreio: str, canal_id: int, transportadora_id: str, transportadora: str, servico_id: int,
                 servico: str, tipo_frete: str, transportadora_fob: Dict, transacao: Dict, usuario: Dict, pagamento: List[Dict], itens: List[Dict], itens_brinde: List[Dict],
                 itens_personalizados: List[Dict], forma_pagamento_parceiro: Dict, forma_envio_parceiro: Dict, pedido_nota_fiscal: List[Dict], protocolo: str, representante: str) -> None:
        self.order_id = id
        self.sale_order_id = pedido_venda_id
        self.order_date = data_pedido
        self.generated_date = data_geracao
        self.erp_code = codigo_referencia
        self.total_amount = valor_total
        self.payment_method_amount = valor_forma_pagamento
        self.discount_amount = valor_desconto
        self.freight_amount = valor_frete
        self.status = status
        self.bonds_quantity = quantidade_titulos
        self.delivery_forecast = previsao_entrega
        self.tracking_code = codigo_rastreio
        self.channel_id = canal_id
        self.carrier_id = transportadora_id
        self.carrier = transportadora
        self.service_id = servico_id
        self.service = servico
        self.freight_type = tipo_frete
        self.fob_carrier = TransportadoraFob(**transportadora_fob) if transportadora_fob is not None else None
        self.transaction = Transacao(**transacao) if transacao is not None else None
        self.user = Usuario(**usuario) if usuario is not None else None
        self.payments = [Pagamento(**p) for p in pagamento] if pagamento is not None else list()
        self.items = [Item(**i) for i in itens] if itens is not None else list()
        self.gift_items = [ItensBrinde(**ib) for ib in itens_brinde] if itens_brinde is not None else list()
        self.custom_items = [ItensPersonalizado(**ip) for ip in itens_personalizados] if itens_personalizados is not None else list()
        self.partner_payment_type = FormaPagamentoParceiro(**forma_pagamento_parceiro) if forma_pagamento_parceiro is not None else None
        self.partner_shipping_method = FormaEnvioParceiro(**forma_envio_parceiro) if forma_envio_parceiro is not None else None
        self.invoices = [PedidoNotaFiscal(**pnf) for pnf in pedido_nota_fiscal] if pedido_nota_fiscal is not None else list()
        self.protocol = protocolo
        self.erp_representative = representante

        if len(self.payments) > 0 and len(self.payments[0].bonds) > 0:
            self.paid_date = self.payments[0].bonds[0].paid_date
            self.payment_type = self.payments[0].payment_option
            self.flag = self.payments[0].flag
            self.installments = self.payments[0].installments
            self.erp_payment_condition = self.payments[0].erp_payment_condition



from datetime import datetime
import logging
from typing import List
import src.database.connection as database
from src.database import queries
from src.database.utils import DatabaseConfig
import src.database.utils as utils
from src.entities.product import Product
from src.entities.photos_sku import PhotosSku
import src.api.okvendas as api_okvendas
from src.jobs.system_jobs import OnlineLogger

logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_update_products_semaphore(job_config_dict: dict):
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para criar produtos nao encontrado', 'warning', 'PRODUTO')
            return

        db = database.Connection(db_config)
        connection = db.get_conect()
        cursor = connection.cursor()

        cursor.execute(db_config.sql)
        cursor.close()

        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produtos marcados para atualizar no banco semaforo: {cursor.rowcount}', 'info', 'PRODUTO')
        connection.commit()
        connection.close()

    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao atualizar produtos no banco semaforo: {str(e)}', 'error', 'PRODUTO')


def job_insert_products_semaphore(job_config_dict: dict):
    """
    Job que realiza a insercao dos produtos no banco semaforo
    Args:
        job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir produtos no semaforo nao encontrado', 'warning', 'PRODUTO')
            return

        # abre a connection com o banco
        db = database.Connection(db_config)
        connection = db.get_conect()
        cursor = connection.cursor()

        try:
            # sql: str = db_config.sql
            # if db_config.sql[len(db_config.sql) - 1] is ';':
            #     sql = db_config.sql[len(db_config.sql) - 1] = ' '

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Inserindo produtos no banco semáforo', 'info', 'PRODUTO')
            cursor.execute(db_config.sql)
            connection.commit()
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{cursor.rowcount} produtos inseridos no banco semáforo', 'info', 'PRODUTO')
        except Exception as e:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao inserir produtos no banco semaforo: {str(e)}', 'error', 'PRODUTO')
            cursor.close()
            connection.close()

        cursor.close()
        connection.close()
    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao durante a execucao do job: {str(e)}', 'error', 'PRODUTO')


def job_send_products(job_config_dict: dict):
    """
    Job que realiza a leitura dos produtos contidos no banco semaforo e envia para a api okvendas
    Args:
        job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para criar produtos nao encontrado', 'warning', 'PRODUTO')
            return

        produtos_repetidos = query_products(db_config)

        keys_photos_sent = {}
        photos_sku = mount_list_photos(produtos_repetidos)
        produtos = remove_repeat_products(produtos_repetidos)

        if len(produtos) <= 0:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao existem produtos para criar no momento', 'warning', 'PRODUTO')
            return

        for prod in produtos:
            try:
                response = api_okvendas.post_produtos([prod])

                for res in response:
                    if res.status > 1:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                      f'Erro ao gerar produto {res.codigo_erp} na api okvendas. Erro gerado na api: {res.message}', 'warning', 'PRODUTO')
                    else:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produto {res.codigo_erp} criado com sucesso', 'info', 'PRODUTO')
                        protocol_products(job_config_dict, produtos, db_config)

                        keys_photos_sent[(prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo)] = ""

            except Exception as e:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro durante o envio de produtos: {str(e)}', 'error', 'PRODUTO')

        if send_photos_products(photos_sku, keys_photos_sent, 50):
            logger.info(job_config_dict.get('job_name') + f' | Fotos dos produtos cadastradas com sucesso')
        else:
            print(" Erro ao cadastrar as fotos do produto")
            logger.error(job_config_dict.get('job_name') + f' | Erro durante o cadastro das fotos dos produtos')

    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro durante a execucao do job: {str(e)}', 'error', 'PRODUTO')


def query_products(db_config: DatabaseConfig):
    """
    Consulta os produtos contidos no banco semaforo juntamente com os dados do banco do ERP
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de produtos
    """
    # abre a connection com o banco
    db = database.Connection(db_config)
    connection = db.get_conect()
    # connection.start_transaction()
    cursor = connection.cursor()

    # obtem os dados do banco
    # logger.warning(query)
    cursor.execute(db_config.sql)
    columns = [col[0] for col in cursor.description]
    results = [dict(zip(columns, row)) for row in cursor.fetchall()]
    cursor.close()
    connection.close()

    produtos = []
    for result in results:
        produtos.append(Product(result))

    return produtos


def protocol_products(job_config_dict: dict, products: list, db_config: DatabaseConfig) -> None:
    """
    Protocola no banco semaforo os produtos que foram enviados para a api okvendas
    Args:
        job_config_dict: Configuração do job
        products: Lista de produtos enviados para a api okvendas
        db_config: Configuracao do banco de dados
    """
    try:
        if len(products) > 0:
            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()
            for prod in products:
                try:
                    dados_produto = [prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo]
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Protocolando codigo_erp {dados_produto[0]} sku {dados_produto[1]}', 'info', 'PRODUTO')
                    cursor.execute(queries.get_product_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, dados_produto))
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Linhas afetadas {cursor.rowcount}', 'info', 'PRODUTO')
                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao protocolar sku {prod["codigo_erp_sku"]}: {str(e)}', 'error', 'PRODUTO')
            cursor.close()
            connection.commit()
            connection.close()
    except Exception as e:
        raise e


def mount_list_photos(products: List[Product]):
    photos = {}

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in photos:

            length_photos = len(photos[key])

            if contains_photo(photos[key], product.imagem_base64):
                continue
            else:
                order_photo = length_photos + 1
                photo_sku = PhotosSku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{order_photo}', order_photo,
                                      False)
                photos[key].append(photo_sku)

        else:
            # Mount photo
            photo_sku = PhotosSku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{1}', 1,
                                  True)

            photos[key] = [photo_sku]

    return photos


def remove_repeat_products(products: List[Product]):
    product_keys = {}
    result_products = []

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in product_keys:
            continue
        else:
            product_keys[key] = ""
            result_products.append(product)

    return result_products


def send_photos_products(photos_sku: dict, keys_photos_sent: dict, limit_photos_sent: int):
    count_success = 0
    list_photos = []
    for photo_key in photos_sku:

        if photo_key in keys_photos_sent:
            list_photo = photos_sku[photo_key]

            for photo in list_photo:
                if (len(list_photos) + 1) <= limit_photos_sent:
                    list_photos.append(photo)

                if len(list_photos) == limit_photos_sent:
                    success = api_okvendas.put_photos_sku(list_photos)
                    if not success:
                        return success
                    else:
                        count_success = count_success + 1
                        list_photos = []

    if 0 < len(list_photos) <= 50:
        return api_okvendas.put_photos_sku(list_photos)

    return count_success > 0 if True else False


def contains_photo(photos: List[PhotosSku], imagem_base64: str):
    for photo in photos:
        if photo.base64_foto == imagem_base64:
            return True
    return False



from datetime import datetime
import logging
import src.database.connection as database
import src.database.utils as utils
from src.api.entities.lista_preco import ListaPreco, Escopo
from src.database.entities.price_list import DbPriceList, PriceListProduct
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
import src.api.okvendas as api_okvendas
from src.entities.price import Price
from src.database import queries
import src
import time
from typing import List, Dict, Tuple
from src.jobs.system_jobs import OnlineLogger

logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_insert_prices_semaphore(job_config_dict: dict):
    """
    Job para preços no banco semáforo
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir precos no semaforo nao encontrado', 'warning', 'PRECO')
    else:
        db = database.Connection(db_config)
        conn = db.get_conect()
        cursor = conn.cursor()

        try:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Inserindo precos no banco semaforo', 'info', 'PRECO')
            logger.info(db_config.sql)
            cursor.execute(db_config.sql)
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{cursor.rowcount} precos inseridos no banco semaforo', 'info', 'PRECO')
        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro {str(ex)}', 'error', 'PRECO')

        cursor.close()
        conn.commit()
        conn.close()


def job_send_prices(job_config_dict: dict):
    """
    Job para realizar a atualização de preços
    Args:
        job_config_dict: Configuração do job
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir precos no semaforo nao encontrado', 'warning', 'PRECO')
        else:
            prices = query_prices(job_config_dict, db_config)
            if prices is not None and len(prices) > 0:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produtos para atualizar preco {len(prices)}', 'info', 'PRECO')

                for price in prices:
                    try:
                        time.sleep(1)
                        response = api_okvendas.post_prices(src.client_data.get('url_api') + '/catalogo/preco', price, src.client_data.get('token_api'))
                        if response.status == 1 or response.status == "Success":
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Protocolando preço do sku {price.codigo_erp}', 'info', 'PRECO')
                            protocol_price(db_config, price)
                        elif (response.status == 'Error' or response.status > 1) and response.message == "Erro ao atualizar o preço. Produto não existente":
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                     f'Nao foi possivel atualizar o preco do sku {price.codigo_erp}. Erro recebido da Api: {response.message}', 'warning', 'PRECO')
                            protocol_price(db_config, price)
                        else:
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                     f'Nao foi possivel atualizar o preco do sku {price.codigo_erp}. Erro recebido da Api: {response.message}', 'warning', 'PRECO', price.codigo_erp)
                    except Exception as e:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Falha ao atualizar preco do sku {price.codigo_erp}: {str(e)}', 'warning', 'PRECO', price.codigo_erp)
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao existem precos a serem enviados no momento', 'warning', 'PRECO')
    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro durante execucao do job: {str(e)}', 'error', 'PRECO')


def job_prices_list(job_config_dict: dict):
    """
        Job para integrar listas de preço com o okvendas

        Args:
            job_config_dict: Configuração do job
        """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para listas de preco nao encontrado', 'warning', 'PRECO')
        return

    try:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Consultando listas de preco no banco semaforo', 'info', 'PRECO')
        duplicated_db_price_lists = query_price_lists(job_config_dict, db_config)

        # Carrega no dict todas as listas de preço agrupando pelo codigo da lista de preco e incrementando a lista de clientes
        unique_api_price_lists = dict.fromkeys(set([d.price_list_code for d in duplicated_db_price_lists]))
        unique_db_price_lists = dict.fromkeys(set([d.price_list_code for d in duplicated_db_price_lists]))
        for d in duplicated_db_price_lists:
            if unique_db_price_lists[d.price_list_code] is None:
                unique_db_price_lists[d.price_list_code] = d

            if unique_api_price_lists[d.price_list_code] is None:
                unique_api_price_lists[d.price_list_code] = ListaPreco(d.price_list_description, d.price_list_code, d.branch_code, d.initial_date, d.final_date,
                                                                       d.active, d.priority, Escopo(d.scope_type, [d.client_code]), d.calculate_ipi)
            else:
                if not unique_api_price_lists[d.price_list_code].escopo.codigos_escopo.__contains__(d.client_code):
                    unique_api_price_lists[d.price_list_code].escopo.codigos_escopo.append(d.client_code)

        db_price_lists = list(unique_db_price_lists.values())
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Inserindo/atualizando listas de preco no banco semaforo', 'info', 'PRECO')
        if not insert_update_semaphore_price_lists(job_config_dict, db_config, db_price_lists):
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao foi possivel inserir as listas de preco no banco semaforo', 'error', 'PRECO')
            return

        api_price_lists = list(unique_api_price_lists.values())
        response_list = api_okvendas.put_price_lists(api_price_lists)
        for res in response_list:
            if res.status != 1:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao processar lista de preco {res.identifiers}: {res.message}', 'warning', 'PRECO')

            if protocol_semaphore_item(job_config_dict, db_config, res.identifiers[0], None):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Lista de preco {res.identifiers[0]} protocolado no banco semaforo', 'info', 'PRECO')
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar a lista de preco {res.identifiers[0]}', 'warning', 'PRECO')

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro {str(ex)}', 'error', 'PRECO')


def job_products_prices_list(job_config_dict: dict):
    """
        Job para inserir a relação de produtos com as listas de preço no banco semáforo

        Args:
            job_config_dict: Configuração do job
        """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir a relação de produtos com as listas de preco no semaforo nao encontrado', 'warning', 'PRECO')
        return

    try:
        db_price_list_products = query_price_list_products(job_config_dict, db_config)

        if not insert_update_semaphore_price_lists_products(job_config_dict, db_config, db_price_list_products):
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao foi possivel inserir os produtos das listas de preco no banco semaforo', 'error', 'PRECO')
            return

        split_price_lists = dict.fromkeys(set([d.price_list_code for d in db_price_list_products]), [])
        for prod in db_price_list_products:
            split_price_lists[prod.price_list_code].append(PriceListProduct(prod.price_list_code, prod.erp_code, prod.price, prod.price_from, prod.other_price))

        for k, v in split_price_lists.items():
            products_file = mount_price_list_products_file(v)
            if not api_okvendas.put_price_lists_products(k, products_file):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao processar lista de preco {k}', 'warning', 'PRECO')

            for prod in db_price_list_products:
                if protocol_semaphore_item(job_config_dict, db_config, prod.price_list_code, prod.erp_code):
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produto {prod.erp_code} da lista de preco {prod.price_list_code} protocolado no banco semaforo', 'info', 'PRECO')
                else:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar o produto {prod.erp_code} da lista de preco {prod.price_list_code}', 'warning', 'PRECO')

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro {str(ex)}', 'error', 'PRECO')


def query_prices(job_config_dict: dict, db_config: DatabaseConfig) -> List[Price]:
    """
    Consulta os precos para atualizar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de preços para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            prices = [Price(**p) for p in results]
            return prices

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar precos no banco semaforo: {str(ex)}', 'error', 'PRECO')

    return []


def protocol_price(db_config: DatabaseConfig, price: Price) -> None:
    """
    Protocola os preços no banco semáforo
    Args:
        db_config: Configuração do banco de dados
        price: Lista de skus para protocolar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    sql = queries.get_price_protocol_command(db_config.db_type)
    try:
        cursor.execute(sql, queries.get_command_parameter(db_config.db_type, [price.codigo_erp, price.preco_atual]))
    except Exception as e:
        raise Exception(f'Erro ao protocolar preco do sku no banco semaforo {price.codigo_erp}: {str(e)}')

    cursor.close()
    conn.commit()
    conn.close()


def query_price_lists(job_config_dict: dict, db_config: DatabaseConfig) -> List[DbPriceList]:
    """
    Consulta os listas de precos para consultar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de preços para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            # print(results)
            lists = [DbPriceList(**p) for p in results]
            return lists

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar listas de precos: {str(ex)}', 'error', 'PRECO')

    return []


def insert_update_semaphore_price_lists(job_config_dict: dict, db_config: DatabaseConfig, lists: List[DbPriceList]) -> bool:
    """
    Consulta os listas de precos para consultar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de listas de preço

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.price_list_code, ' ', IntegrationType.LISTA_PRECO.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar listas de precos no banco semaforo: {str(ex)}', 'error', 'PRECO')

    return False


def insert_update_semaphore_price_lists_products(job_config_dict: dict, db_config: DatabaseConfig, lists: List[PriceListProduct]) -> bool:
    """
    Consulta os listas de precos para consultar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de produtos de um(as) listas de preço

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.price_list_code, li.erp_code, IntegrationType.LISTA_PRECO_PRODUTO.value, datetime.now()) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar listas de precos no banco semaforo: {str(ex)}', 'error', 'PRECO')

    return False


def query_price_list_products(job_config_dict: dict, db_config: DatabaseConfig) -> List[PriceListProduct]:
    """
        Consultar no banco semáforo a lista de produtos relacionados a lista de preço

        Args:
            job_config_dict: Configuração do job
            db_config: Configuração do banco de dados

        Returns:
        Lista de produtos relacionados a lista de preço informada
        """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            lists = [PriceListProduct(**p) for p in results]
            return lists

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar produtos da lista de preco no banco semaforo: {str(ex)}', 'error', 'PRECO')

    return []


def mount_price_list_products_file(products: List[PriceListProduct]) -> str:
    file: str = ''
    for p in products:
        file += f'{p.price_list_code},{p.erp_code},{p.price},{p.price_from},{p.other_price}\n'

    return file


def protocol_semaphore_item(job_config_dict: dict, db_config: DatabaseConfig, identifier: str, identifier2: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        if identifier2 is None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        else:
            cursor.execute(queries.get_protocol_semaphore_id2_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier, identifier2]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar item no banco semaforo: {str(ex)}', 'error', 'PRECO')




class PhotosSku:
    def __init__(self, base64_foto: str, codigo_erp_sku: str, codigo_foto: str, ordem: int, foto_padrao: bool):
        self.base64_foto: str = base64_foto
        self.codigo_erp_sku: str = codigo_erp_sku
        self.codigo_foto: str = codigo_foto
        self.ordem: str = ordem
        self.foto_padrao: bool = foto_padrao



class Invoice:
    def __init__(self, pedido_id: int, chave_nf: str, num_serie: str, num_nf: str, valor_nf: float,
                 data_emissao: str, quantidade: int, link_nf: str, link_xml: str, xml: str):
        self.id: int = pedido_id
        self.chave_acesso: str = chave_nf
        self.numero_serie: str = num_serie
        self.numero_nota: str = num_nf
        self.valor_total_nota: float = valor_nf
        self.data_emissao: str = data_emissao
        self.quantidade_volume: int = quantidade
        self.link_danfe: str = link_nf
        self.link_xml: str = link_xml
        self.xml: str = xml



from typing import List


class Escopo:
    def __init__(self, tipo_escopo: int, codigos_escopo: List[str]):
        self.tipo_escopo: int = tipo_escopo
        self.codigos_escopo: List[str] = codigos_escopo


class ListaPreco:
    def __init__(self, descricao_lista_preco: str, codigo_lista_preco: str, codigo_filial: str, data_inicial: str, data_final: str, ativo: bool, prioridade: int, escopo: Escopo, calcula_ipi: str):
        self.descricao_lista_preco: str = descricao_lista_preco
        self.codigo_lista_preco: str = codigo_lista_preco
        self.codigo_filial: str = codigo_filial
        self.data_inicial: str = data_inicial
        self.data_final: str = data_final
        self.ativo: bool = ativo
        self.prioridade: int = prioridade
        self.escopo: Escopo = escopo
        self.calcula_ipi: str = calcula_ipi


class ListaPrecoResponse:
    def __init__(self, Identifiers, Status, Message, Protocolo):
        self.identifiers: List[str] = Identifiers
        self.status: int = Status
        self.message: str = Message
        self.protocol: str = Protocolo



from setuptools import setup, find_packages
from os import path


def read(fname):
    return open(path.join(path.dirname(__file__), fname), encoding='UTF-8').read()


# def save_version(v: str):
#     f = open('version.txt', 'w')
#     f.write(f'\tOking {v} - Openk Tecnologia')
#     f.close()


# Utilizar o padrão x.x.x.xxxx para caso precise subir versão de testes para o respositorio test-pypi
# Utilizar o padrão x.x.x para subir em produção
version = '1.8.10'
package_name = "oking" if len([c for c in version if c == '.']) < 3 else 'okingtest'
# save_version(version)
setup(
    name=package_name,
    version=version,
    author="Openk Tecnologia",
    author_email="<suporte.b2c@openk.com.br>",
    description='Pacote de integração de produtos, preço, estoque e pedidos com o sistema OkVendas da Openk',
    long_description_content_type="text/markdown; charset=UTF-8; variant=GFM",
    long_description=read('project_description.md'),
    packages=find_packages(include=['src', 'src.*']),
    install_requires=['mysql-connector-python',
                      'schedule',
                      'requests',
                      'configparser',
                      'logger',
                      'cx-Oracle',
                      'jsonpickle',
                      'pyodbc'],
    keywords=['python', 'oking', 'openk', 'okvendas', 'ok'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        # "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    entry_points={
        'console_scripts': [f'{package_name}=src.__main__:main']
    }
)



import logging
from datetime import datetime
from time import sleep
from src.entities.log import Log
import src
import src.api.okvendas as api_okvendas
import src.database.connection as database
import src.database.utils as utils
from src.database import queries
from src.database.utils import DatabaseConfig
from src.entities.orderb2b import OrderB2B
from src.entities.tracking import Tracking
from src.entities.invoice import Invoice
from src.entities.order import Order, Queue
from typing import List
from src.jobs.system_jobs import OnlineLogger

logger = logging.getLogger()
send_log = OnlineLogger.send_log

default_limit = 50
queue_status = {
    'pending': 'PEDIDO',
    'paid': 'PEDIDO_PAGO',
    'shipped': 'ENCAMINHADO_ENTREGA',
    'delivered': 'ENTREGUE',
    'canceled': 'CANCELADO',
    'no_invoice': 'SEM_NOTA_FISCAL',
    'invoiced': 'FATURADO'
}


def define_job_start(job_config: dict) -> None:
    global current_job
    current_job = job_config.get('job_name')
    if current_job == 'internaliza_pedidos_job' or current_job == 'internaliza_pedidos_b2b_job':  # Inicia o job a partir dos pedidos AgPagamento
        job_orders(job_config, True)
    elif current_job == 'internaliza_pedidos_pagos_job' or current_job == 'internaliza_pedidos_pagos_b2b_job':  # Inicia o job a partir do pedidos pagos
        job_orders(job_config)


def job_orders(job_config: dict, start_at_pending: bool = False) -> None:
    try:
        db_config = utils.get_database_config(job_config)
        if not current_job.__contains__('b2b'):
            if start_at_pending:
                process_order_queue(job_config, queue_status.get('pending'), db_config, True)

            process_order_queue(job_config, queue_status.get('paid'), db_config, True)

            process_order_queue(job_config, queue_status.get('canceled'), db_config)

        else:
            if start_at_pending:
                process_b2b_order_queue(job_config, queue_status.get('pending'), db_config, True)

            process_b2b_order_queue(job_config, queue_status.get('paid'), db_config, True)

            process_b2b_order_queue(job_config, queue_status.get('canceled'), db_config)

    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro ao inicializar job: {str(e)}', 'error', 'PEDIDO')


def job_invoice_orders(job_config: dict):
    try:
        db_config = utils.get_database_config(job_config)
        if db_config.sql is None:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Comando sql para baixar notas fiscais nao encontrado', 'warning', 'PEDIDO')
        else:
            invoices = query_invoices(job_config, db_config)
            for invoice in invoices:
                try:
                    invoice_sent = api_okvendas.post_invoices(src.client_data.get('url_api') + '/pedido/faturar', invoice, src.client_data.get('token_api'))
                    if invoice_sent is None:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'NF do pedido {invoice.id} enviada com sucesso para api okvendas', 'info', 'PEDIDO')
                        continue
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao enviar NF do pedido {invoice.id} para api okvendas: {invoice_sent.message}', 'error',
                                      'PEDIDO')
                except Exception as e:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao enviar NF do pedido {invoice.id}: {str(e)}', 'error', 'PEDIDO')
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha na execução do job: {str(e)}', 'error', 'PEDIDO')


def job_send_erp_tracking(job_config: dict):
    try:
        db_config = utils.get_database_config(job_config)
        if db_config.sql is None:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Comando sql para consultar rastreios nao encontrado', 'warning', 'PEDIDO')
        else:
            trackings = query_trackings(db_config)
            for tracking in trackings:
                try:
                    tracking_sent = api_okvendas.post_tracking(src.client_data.get('url_api') + '/pedido/encaminhar', tracking, src.client_data.get('token_api'))
                    if tracking_sent is None:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Rastreio do pedido {tracking.id} enviada com sucesso para api okvendas', 'info', 'PEDIDO')
                        continue
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao enviar rastreio do pedido {tracking.id} para api okvendas: {tracking_sent.message}',
                                      'error', 'PEDIDO')
                except Exception as e:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao enviar rastreio do pedido {tracking.id}: {str(e)}', 'error', 'PEDIDO')
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha na execução do job: {str(e)}', 'error', 'PEDIDO')


def process_order_queue(job_config: dict, status: str, db_config: DatabaseConfig, status_to_insert: bool = False) -> None:
    try:
        print()
        print()
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Consultando fila de pedidos no status {status}', 'info', 'PEDIDO')
        queue = api_okvendas.get_order_queue(
            url=src.client_data.get('url_api') + '/pedido/fila/{0}',
            token=src.client_data.get('token_api'),
            status=status,
            limit=default_limit)

        qty = 0
        for q_order in queue:
            try:
                sleep(0.5)
                qty = qty + 1
                print()
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.order_id}', 'info', 'PEDIDO')
                order = api_okvendas.get_order(
                    url=src.client_data.get('url_api') + '/pedido/{0}',
                    token=src.client_data.get('token_api'),
                    order_id=q_order.order_id
                )

                if order.erp_code is not None and order.erp_code != '':  # Pedido integrado anteriormente

                    if check_order_existence(db_config, order.order_id):
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido ja integrado com o ERP, chamando procedure de atualizacao...', 'info', 'PEDIDO')
                        if call_update_order_procedure(job_config, db_config, order):
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido atualizado com sucesso', 'info', 'PEDIDO')
                            protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id='', client_erp_id='')
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                      f'Pedido {order.order_id} nao existe no banco semaforo porem ja foi integrado previamente. Protocolando pedido...', 'warning', 'PEDIDO')
                        protocol_non_existent_order(job_config, q_order)

                else:  # Pedido nao integrado anteriormente

                    if check_non_integrated_order_existence(db_config, order.order_id):  # Pedido existente no banco semaforo

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido já existente no banco semáforo, porem nao integrado com erp. Chamando procedures',
                                      'info', 'PEDIDO')
                        sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config, q_order.order_id)
                        if sp_success:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Chamadas das procedures executadas com sucesso, protocolando pedido...', 'info', 'PEDIDO')
                            protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    elif status_to_insert:  # Pedido nao existe no semaforo e esta em status de internalizacao (pending e paid)

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Inserindo novo pedido no banco semaforo', 'info', 'PEDIDO')
                        inserted = insert_temp_order(job_config, order, db_config)
                        if inserted:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido inserido com sucesso, chamando procedures...', 'info', 'PEDIDO')
                            sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config, q_order.order_id)
                            if sp_success:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Chamadas das procedures executadas com sucesso, protocolando pedido...', 'info',
                                              'PEDIDO')
                                protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    else:  # Pedido nao existe no semaforo e nao esta em status de internalizacao (pending e paid)
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido nao existente no banco semaforo e nao se encontra em status de internalizacao',
                                      'warning', 'PEDIDO')

            except Exception as e:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro no processamento do pedido {q_order.order_id}: {str(e)}', 'error', 'PEDIDO', q_order.order_id)
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}', 'error', 'PEDIDO')


def process_b2b_order_queue(job_config: dict, status: str, db_config: DatabaseConfig, status_to_insert: bool = False) -> None:
    try:
        print()
        print()
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Consultando fila de pedidos no status {status}', 'info', 'PEDIDO')
        queue = api_okvendas.get_order_queue(
            url=src.client_data.get('url_api') + '/pedido/fila/{0}',
            token=src.client_data.get('token_api'),
            status=status,
            limit=default_limit)

        qty = 0
        for q_order in queue:
            try:
                sleep(0.5)
                qty = qty + 1
                print()
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.order_id}', 'info', 'PEDIDO')
                order = api_okvendas.get_order_b2b(
                    url=src.client_data.get('url_api') + '/pedidoB2B/{0}',
                    token=src.client_data.get('token_api'),
                    order_id=q_order.order_id
                )

                if order.erp_code is not None and order.erp_code != '':  # Pedido integrado anteriormente

                    if check_order_existence(db_config, order.order_id):
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido ja integrado com o ERP, chamando procedure de atualizacao...', 'info', 'PEDIDO')
                        if call_update_order_procedure(job_config, db_config, order):
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido atualizado com sucesso', 'info', 'PEDIDO')
                            protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id='', client_erp_id='')
                    else:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False,
                                      f'Pedido {order.order_id} nao existe no banco semaforo porem ja foi integrado previamente. Protocolando pedido...', 'warning', 'PEDIDO')
                        protocol_non_existent_order(job_config, q_order)

                else:  # Pedido nao integrado anteriormente

                    if check_order_existence(db_config, order.order_id):  # Pedido existente no banco semaforo

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido já existente no banco semáforo, porem nao integrado com erp. Chamando procedures',
                                      'info', 'PEDIDO')
                        sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config, q_order.order_id)
                        if sp_success:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Chamadas das procedures executadas com sucesso, protocolando pedido...', 'info', 'PEDIDO')
                            protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    elif status_to_insert:  # Pedido nao existe no semaforo e esta em status de internalizacao (pending e paid)

                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Inserindo novo pedido no banco semaforo', 'info', 'PEDIDO')
                        inserted = insert_temp_b2b_order(job_config, order, db_config)
                        if inserted:
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido inserido com sucesso, chamando procedures...', 'info', 'PEDIDO')
                            sp_success, client_erp_id, order_erp_id = call_order_procedures(job_config, db_config, q_order.order_id)
                            if sp_success:
                                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Chamadas das procedures executadas com sucesso, protocolando pedido...', 'info',
                                              'PEDIDO')
                                protocol_order(job_config, db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)

                    else:  # Pedido nao existe no semaforo e nao esta em status de internalizacao (pending e paid)
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido nao existente no banco semaforo e nao se encontra em status de internalizacao',
                                      'warning', 'PEDIDO')

            except Exception as e:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro no processamento do pedido {q_order.order_id}: {str(e)}', 'error', 'PEDIDO', q_order.order_id)
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}', 'error', 'PEDIDO')


def protocol_order(job_config: dict, db_config: DatabaseConfig, order, queue_order: Queue, order_erp_id: str = '', client_erp_id: str = '') -> None:
    db = database.Connection(db_config)
    try:
        if order_erp_id != '':
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Protocolando pedido com codigo_referencia {order_erp_id}', 'info', 'PEDIDO')
            updated_order_code = api_okvendas.put_order_erp_code(src.client_data.get('url_api') + '/pedido/integradoERP',
                                                                 src.client_data.get('token_api'),
                                                                 order.order_id,
                                                                 order_erp_id)
            if updated_order_code:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Codigo Erp do pedido atualizado via api OkVendas', 'info', 'PEDIDO')
            else:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao atualizar o Codigo Erp do pedido via api OkVendas', 'warning', 'PEDIDO')

        if client_erp_id != '':
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Salvando codigo do erp do cliente no banco semaforo {client_erp_id}', 'info', 'PEDIDO')
            conn = db.get_conect()
            cursor = conn.cursor()
            cursor.execute(queries.get_client_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [client_erp_id, order.order_id]))
            cursor.close()
            conn.commit()
            conn.close()

            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Protocolando cliente via api okvendas com codigo_referencia {client_erp_id}', 'info', 'PEDIDO')
            updated_client_code = api_okvendas.put_client_erp_code(src.client_data.get('url_api') + '/cliente/codigo',
                                                                   src.client_data.get('token_api'),
                                                                   {
                                                                       'cpf_cnpj': order.user.cpf if order.user.cpf is not None or order.user.cpf != '' else order.user.cnpj,
                                                                       'codigo_cliente': client_erp_id
                                                                   })
            if updated_client_code:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Codigo Erp do cliente atualizado via api OkVendas', 'info', 'PEDIDO')
            else:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao atualizar o Codigo Erp do cliente via api OkVendas', 'warning', 'PEDIDO')

        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Removendo pedido da fila pelo protocolo {queue_order.protocol}', 'info', 'PEDIDO')
        protocoled_order = api_okvendas.put_protocol_orders([queue_order.protocol])
        if protocoled_order:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido protocolado via api OkVendas', 'info', 'PEDIDO')
        else:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao protocolar pedido via api OkVendas', 'warning', 'PEDIDO')

        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Protocolando pedido no banco semaforo', 'info', 'PEDIDO')
        conn = db.get_conect()
        cursor = conn.cursor()
        cursor.execute(queries.get_order_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order_erp_id, order.order_id]))
        cursor.close()
        conn.commit()
        conn.close()
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro ao protocolar pedidos: {str(e)}', 'error', 'PEDIDO', order.order_id)


def protocol_non_existent_order(job_config: dict, queue_order: Queue) -> None:
    try:
        protocoled_order = api_okvendas.put_protocol_orders([queue_order.protocol])
        if protocoled_order:
            logger.info(current_job + f' | Pedido {queue_order.order_id} protocolado via api OkVendas')
        else:
            logger.warning(current_job + f' | Falha ao protocolar pedido {queue_order.order_id} via api OkVendas')
    except Exception as ex:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro ao protocolar pedido {queue_order.order_id}: {str(ex)}', 'error', 'PEDIDO', queue_order.order_id)


def insert_temp_order(job_config: dict, order: Order, db_config: DatabaseConfig) -> bool:
    step = ''
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        step = 'conexao'
        cursor = conn.cursor()

        existent_client = None
        if order.user.erp_code is not None and order.user.erp_code != '' and order.user.erp_code != '0':  # Por padrao o erp_code vem = '0' da api okvendas
            cursor.execute(queries.get_query_client_erp(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
            existent_client = cursor.fetchone()

        if existent_client is None:
            # insere cliente
            step = 'insere cliente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Inserindo cliente', 'info', 'PEDIDO')
            cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order.user.name or order.user.company_name,
                order.user.company_name or order.user.name,
                order.user.cpf,
                order.user.cnpj,
                order.user.email,
                order.user.residential_phone,
                order.user.mobile_phone,
                order.user.address.zipcode,
                order.user.address.address_type,
                order.user.address.address_line,
                order.user.address.number,
                order.user.address.complement,
                order.user.address.neighbourhood or " ",
                order.user.address.city,
                order.user.address.state,
                order.user.address.reference,
                'IN'
            ]))
        else:
            # update no cliente existente
            step = 'update cliente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Atualizando cliente existente', 'info', 'PEDIDO')
            cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order.user.name,
                order.user.company_name or order.user.name,
                order.user.cpf,
                order.user.cnpj,
                order.user.email,
                order.user.residential_phone,
                order.user.mobile_phone,
                order.user.address.zipcode,
                order.user.address.address_type,
                order.user.address.address_line,
                order.user.address.number,
                order.user.address.complement,
                order.user.address.neighbourhood,
                order.user.address.city,
                order.user.address.state,
                order.user.address.reference,
                order.user.erp_code
            ]))

        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Cliente inserido/atualizado para o pedido {order.order_id}', 'info', 'PEDIDO')
            cursor.execute(queries.get_query_client_id(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
            (client_id, ) = cursor.fetchone()
            if client_id is None or client_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere pedido
        step = 'insere pedido'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Inserindo cabecalho pedido', 'info', 'PEDIDO')
        cursor.execute(queries.get_insert_order_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
            order.order_id,
            order.order_code,
            str(datetime.strptime(order.date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.date is not None else ''),
            order.status,
            client_id,
            order.total_amount,
            order.total_discount,
            order.freight_amount,
            order.additional_payment_amount,
            str(datetime.strptime(order.paid_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.paid_date is not None else ''),
            order.payment_type,
            order.flag,
            order.parcels,
            order.erp_payment_condition,
            order.tracking_code,
            str(datetime.strptime(order.delivery_forecast.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.delivery_forecast is not None else ''),
            order.carrier,
            order.shipping_mode,
            order.channel_id
        ]))

        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido {order.order_id} inserido', 'info', 'PEDIDO')
            cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.order_id]))
            (order_id, ) = cursor.fetchone()
            if order_id is None or order_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere itens
        step = 'insere itens'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id} com id semaforo {order_id}: Inserindo itens do pedido', 'info', 'PEDIDO')
        for item in order.items:
            cursor.execute(queries.get_insert_order_items_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order_id,
                item.sku,
                item.erp_code,
                item.quantity,
                item.ean,
                item.value,
                item.discount,
                item.freight_value
            ]))

        cursor.close()
        conn.commit()
        conn.close()
        return True
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}', 'error', 'PEDIDO',
                      str(order.order_id))
        conn.rollback()
        conn.close()
        return False


def insert_temp_b2b_order(job_config: dict, order: OrderB2B, db_config: DatabaseConfig) -> bool:
    step = ''
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        step = 'conexao'
        cursor = conn.cursor()

        existent_client = None
        if order.user.erp_code is not None and order.user.erp_code != '' and order.user.erp_code != '0':  # Por padrao o erp_code vem = '0' da api okvendas
            cursor.execute(queries.get_query_client_erp(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
            existent_client = cursor.fetchone()

        if existent_client is None:
            # insere cliente
            step = 'insere cliente'
            logger.info(f'')
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Inserindo cliente', 'info', 'PEDIDO')
            cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order.user.name or order.user.company_name,
                order.user.company_name or order.user.name,
                order.user.cpf,
                order.user.cnpj,
                order.user.email,
                order.user.residential_phone,
                order.user.mobile_phone,
                order.user.address.zipcode,
                order.user.address.address_type,
                order.user.address.address_line,
                order.user.address.number,
                order.user.address.complement,
                order.user.address.neighbourhood or " ",
                order.user.address.city,
                order.user.address.state,
                order.user.address.reference,
                'IN'
            ]))
        else:
            # update no cliente existente
            step = 'update cliente'
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Atualizando cliente existente', 'info', 'PEDIDO')
            cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order.user.name,
                order.user.company_name or order.user.name,
                order.user.cpf,
                order.user.cnpj,
                order.user.email,
                order.user.residential_phone,
                order.user.mobile_phone,
                order.user.address.zipcode,
                order.user.address.address_type,
                order.user.address.address_line,
                order.user.address.number,
                order.user.address.complement,
                order.user.address.neighbourhood,
                order.user.address.city,
                order.user.address.state,
                order.user.address.reference,
                order.user.erp_code
            ]))

        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Cliente inserido/atualizado para o pedido {order.order_id}', 'info', 'PEDIDO')
            cursor.execute(queries.get_query_client_id(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
            (client_id, ) = cursor.fetchone()
            if client_id is None or client_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere pedido
        step = 'insere pedido'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id}: Inserindo cabecalho pedido', 'info', 'PEDIDO')
        cursor.execute(queries.get_insert_b2b_order_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
            order.order_id,
            order.sale_order_id,
            str(datetime.strptime(order.order_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.order_date is not None else ''),
            order.status,
            client_id,
            order.total_amount,
            order.discount_amount,
            order.freight_amount,
            0.0,
            str(datetime.strptime(order.payments[0].bonds[0].paid_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if len(order.payments) > 0 and len(order.payments[0].bonds) > 0 and order.paid_date is not None else ''),
            order.payment_type,
            order.flag,
            order.installments,
            order.erp_payment_condition,
            order.tracking_code,
            str(datetime.strptime(order.delivery_forecast.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.delivery_forecast is not None else ''),
            order.carrier,
            '',
            order.channel_id,
            order.erp_representative
        ]))

        if cursor.rowcount > 0:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido {order.order_id} inserido', 'info', 'PEDIDO')
            cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.order_id]))
            (order_id, ) = cursor.fetchone()
            if order_id is None or order_id <= 0:
                cursor.close()
                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere itens
        step = 'insere itens'
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'\tPedido {order.order_id} com id semaforo {order_id}: Inserindo itens do pedido', 'info', 'PEDIDO')
        for item in order.items:
            cursor.execute(queries.get_insert_b2b_order_items_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order_id,
                item.sku,
                item.erp_code,
                item.quantity,
                item.ean,
                item.value,
                item.discount,
                0,  # Valor do frete não necessario, usar o do cabeçalho
                item.selling_branch_cnpj,
                item.branch_erp_code,
                item.expedition_branch,
                item.invoice_branch
            ]))

        cursor.close()
        conn.commit()
        conn.close()
        return True
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}', 'error', 'PEDIDO'
                      , str(order.order_id))
        conn.rollback()
        conn.close()
        return False


def call_order_procedures(job_config: dict, db_config: DatabaseConfig, order_id: int) -> (bool, str, str):
    client_erp_id = ''
    order_erp_id = ''
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Executando procedure de cliente', 'info', 'PEDIDO')
        if db_config.is_sql_server():
            cursor.execute('exec openk_semaforo.sp_processa_cliente @pedido = ?', order_id)
            (client_erp_id, ) = cursor.fetchone()
        elif db_config.is_oracle() or db_config.is_mysql():
            client_out_value = cursor.var(str)
            cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_CLIENTE', [order_id, client_out_value])
            client_erp_id = client_out_value.getvalue()

        if client_erp_id is not None:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Cliente ERP criado com sucesso {client_erp_id}', 'info', 'PEDIDO')
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Executando procedure de pedido', 'info', 'PEDIDO')
            if db_config.is_sql_server():
                cursor.execute('exec openk_semaforo.sp_processa_pedido @pedido = ?', order_id)
                (order_erp_id,) = cursor.fetchone()
            elif db_config.is_oracle() or db_config.is_mysql():
                order_out_value = cursor.var(str)
                cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_PEDIDO', [order_id, int(client_erp_id), order_out_value])
                order_erp_id = order_out_value.getvalue()

            if order_erp_id is not None:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Pedido ERP criado com sucesso {order_erp_id}', 'info', 'PEDIDO')
            else:
                success = False
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nao foi possivel obter o id do pedido do ERP (retorno da procedure)', 'warning', 'PEDIDO')
        else:
            success = False
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nao foi possivel obter o id do cliente do ERP (retorno da procedure)', 'warning', 'PEDIDO')

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success, client_erp_id, order_erp_id
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro no método de chamada da procedure de internalização do pedido {order_id}: {str(e)}', 'error', 'PEDIDO',
                      str(order_id))
        conn.rollback()
        conn.close()


def call_update_order_procedure(job_config: dict, db_config: DatabaseConfig, order) -> bool:
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        updated: bool = False
        if db_config.is_sql_server():
            cursor.execute('exec openk_semaforo.sp_atualiza_pedido @pedido = ?', order.order_id)
            (updated, ) = cursor.fetchone()
        elif db_config.is_oracle() or db_config.is_mysql():
            order_updated = cursor.var(int)
            cursor.callproc('OPENK_SEMAFORO.SP_ATUALIZA_PEDIDO', [order.order_id, order.status, order_updated])
            updated = order_updated.getvalue()
        if updated is None or updated <= 0:
            success = False
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nao foi possivel atualizar o pedido informado', 'warning', 'PEDIDO')

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success
    except Exception as e:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), True, f'Erro no método de chamada da procedure de atualização do pedido {order.order_id}: {str(e)}', 'error',
                      'PEDIDO', str(order.order_id))
        conn.rollback()
        conn.close()


def check_order_existence(db_config: DatabaseConfig, order_id: int) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order_id]))
        existent_order = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        return existent_order is not None and existent_order > 0
    except Exception as e:
        conn.close()
        return False


def check_non_integrated_order_existence(db_config: DatabaseConfig, order_id: int) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        cursor.execute(queries.get_query_non_integrated_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order_id]))
        existent_order = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        return existent_order is not None and existent_order > 0
    except Exception as e:
        conn.close()
        return False


def query_invoices(job_config: dict, db_config: DatabaseConfig) -> List[Invoice]:
    """
    Consulta as notas fiscais a serem enviadas na api do okvendas
    Args:
        job_config: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de notas fiscais para enviar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    invoices = List[Invoice]
    try:
        cursor.execute(db_config.sql.replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            invoices = [Invoice(**p) for p in results]

    except Exception as ex:
        logger.error(f' ')
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro ao consultar notas fiscais no banco: {str(ex)}', 'error', 'PEDIDO')

    return invoices


def query_trackings(job_config: dict, db_config: DatabaseConfig) -> List[Tracking]:
    """
    Consulta os rastreios a serem enviados na api do okvendas
    Args:
        job_config: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de notas fiscais para enviar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    trackings = List[Tracking]
    try:
        cursor.execute(db_config.sql.replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            trackings = [Tracking(**p) for p in results]

    except Exception as ex:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro ao consultar rastreios no banco: {str(ex)}', 'error', 'PEDIDO')

    return trackings





class DbPriceList:
    def __init__(self, ativo, codigo_filial, codigo_lista_preco, data_final, data_inicial, descricao_lista_preco, prioridade, tipo_escopo, cliente_erp, calcula_ipi='S'):
        self.active: bool = ativo
        self.branch_code: str = codigo_filial
        self.price_list_code: str = codigo_lista_preco
        self.final_date: str = data_final
        self.initial_date: str = data_inicial
        self.price_list_description: str = descricao_lista_preco
        self.priority: int = prioridade
        self.scope_type: int = tipo_escopo
        self.client_code: str = cliente_erp
        self.calculate_ipi: str = calcula_ipi


class PriceListProduct:
    def __init__(self, codigo_lista_preco, codigo_erp, preco_atual, preco_lista, preco_outro):
        self.price_list_code: str = codigo_lista_preco
        self.erp_code: str = codigo_erp
        self.price: float = preco_atual
        self.price_from: float = preco_lista
        self.other_price: float = preco_outro



import pyodbc

conn = pyodbc.connect('Driver={SQL Server};'
                      'Server=server_name;'
                      'Database=database_name;'
                      'Trusted_Connection=yes;')

cursor = conn.cursor()
cursor.execute('SELECT * FROM openk_semaforo.produto')

for row in cursor:
    print(row)




class Log:
    def __init__(self, error: str, date_time: str, identifier: str, method: str):
        self.erro: str = error[0:2000] if error is not None else ''
        self.data_hora: str = date_time
        self.identificador: str = identifier
        self.metodo: str = method
